@@ -13,7 +13,6 @@
 #    under the License.
 
 from oslo_log import log as logging
-
 import pecan
 import six
 
@@ -67,11 +66,13 @@ class CapsuleCollection(collection.Collection):
 
     @staticmethod
     def convert_with_links(rpc_capsules, limit, url=None,
-                           expand=False, **kwargs):
+                           expand=False, legacy_api_version=False, **kwargs):
         context = pecan.request.context
         collection = CapsuleCollection()
         collection.capsules = \
-            [view.format_capsule(url, p, context) for p in rpc_capsules]
+            [view.format_capsule(url, p, context,
+                                 legacy_api_version=legacy_api_version)
+             for p in rpc_capsules]
         collection.next = collection.get_next(limit, url=url, **kwargs)
         return collection
 
@@ -83,17 +84,24 @@ class CapsuleController(base.Controller):
 
     }
 
+    @base.Controller.api_version("1.1", "1.31")
     @pecan.expose('json')
     @exception.wrap_pecan_controller_exception
     def get_all(self, **kwargs):
         '''Retrieve a list of capsules.'''
+        return self._do_get_all(legacy_api_version=True, **kwargs)
+
+    @base.Controller.api_version("1.32")  # noqa
+    @pecan.expose('json')
+    @exception.wrap_pecan_controller_exception
+    def get_all(self, **kwargs):
+        '''Retrieve a list of capsules.'''
+        return self._do_get_all(**kwargs)
+
+    def _do_get_all(self, legacy_api_version=False, **kwargs):
         context = pecan.request.context
         policy.enforce(context, "capsule:get_all",
                        action="capsule:get_all")
-        return self._get_capsules_collection(**kwargs)
-
-    def _get_capsules_collection(self, **kwargs):
-        context = pecan.request.context
         if utils.is_all_projects(kwargs):
             context.all_projects = True
         limit = api_utils.validate_limit(kwargs.get('limit'))
@@ -114,12 +122,12 @@ class CapsuleController(base.Controller):
                                         sort_dir,
                                         filters=filters)
 
-        return CapsuleCollection.convert_with_links(capsules, limit,
-                                                    url=resource_url,
-                                                    expand=expand,
-                                                    sort_key=sort_key,
-                                                    sort_dir=sort_dir)
+        return CapsuleCollection.convert_with_links(
+            capsules, limit, url=resource_url, expand=expand,
+            sort_key=sort_key, sort_dir=sort_dir,
+            legacy_api_version=legacy_api_version)
 
+    @base.Controller.api_version("1.1", "1.31")
     @pecan.expose('json')
     @api_utils.enforce_content_types(['application/json'])
     @exception.wrap_pecan_controller_exception
@@ -129,6 +137,21 @@ class CapsuleController(base.Controller):
 
         :param capsule_dict: a capsule within the request body.
         """
+        return self._do_post(legacy_api_version=True, **capsule_dict)
+
+    @base.Controller.api_version("1.32")  # noqa
+    @pecan.expose('json')
+    @api_utils.enforce_content_types(['application/json'])
+    @exception.wrap_pecan_controller_exception
+    @validation.validated(schema.capsule_create)
+    def post(self, **capsule_dict):
+        """Create a new capsule.
+
+        :param capsule_dict: a capsule within the request body.
+        """
+        return self._do_post(**capsule_dict)
+
+    def _do_post(self, legacy_api_version=False, **capsule_dict):
         context = pecan.request.context
         compute_api = pecan.request.compute_api
         policy.enforce(context, "capsule:create",
@@ -140,14 +163,15 @@ class CapsuleController(base.Controller):
         spec_content, template_json = \
             utils.check_capsule_template(capsules_template)
 
-        containers_spec = utils.capsule_get_container_spec(spec_content)
+        containers_spec, init_containers_spec = \
+            utils.capsule_get_container_spec(spec_content)
         volumes_spec = utils.capsule_get_volume_spec(spec_content)
 
         # Create the capsule Object
         new_capsule = objects.Capsule(context, **capsule_dict)
         new_capsule.project_id = context.project_id
         new_capsule.user_id = context.user_id
-        new_capsule.status = consts.PENDING
+        new_capsule.status = consts.CREATING
         new_capsule.create(context)
         new_capsule.volumes = []
         capsule_need_cpu = 0
@@ -163,7 +187,7 @@ class CapsuleController(base.Controller):
                                                        'always')
         container_restart_policy = {"MaximumRetryCount": "0",
                                     "Name": capsule_restart_policy}
-        new_capsule.restart_policy = capsule_restart_policy
+        new_capsule.restart_policy = container_restart_policy
 
         metadata_info = template_json.get('metadata', None)
         requested_networks_info = template_json.get('nets', [])
@@ -171,8 +195,8 @@ class CapsuleController(base.Controller):
             utils.build_requested_networks(context, requested_networks_info)
 
         if metadata_info:
-            new_capsule.meta_name = metadata_info.get('name', None)
-            new_capsule.meta_labels = metadata_info.get('labels', None)
+            new_capsule.name = metadata_info.get('name', None)
+            new_capsule.labels = metadata_info.get('labels', None)
 
         # create the capsule in DB so that it generates a 'id'
         new_capsule.save()
@@ -182,22 +206,11 @@ class CapsuleController(base.Controller):
         if az_info:
             extra_spec['availability_zone'] = az_info
 
-        # Generate Object for infra container
-        sandbox_container = objects.Container(context)
-        sandbox_container.project_id = context.project_id
-        sandbox_container.user_id = context.user_id
-        name = self._generate_name_for_capsule_sandbox(new_capsule)
-        sandbox_container.name = name
-        sandbox_container.capsule_id = new_capsule.id
-        sandbox_container.image = CONF.sandbox_image
-        sandbox_container.image_driver = CONF.sandbox_image_driver
-        sandbox_container.image_pull_policy = \
-            CONF.sandbox_image_pull_policy
-        sandbox_container.status = consts.CREATING
-        sandbox_container.create(context)
-        new_capsule.containers_uuids = [sandbox_container.uuid]
+        new_capsule.image = CONF.sandbox_image
+        new_capsule.image_driver = CONF.sandbox_image_driver
 
-        for container_spec in containers_spec:
+        merged_containers_spec = init_containers_spec + containers_spec
+        for container_spec in merged_containers_spec:
             container_dict = container_spec
             container_dict['project_id'] = context.project_id
             container_dict['user_id'] = context.user_id
@@ -230,20 +243,28 @@ class CapsuleController(base.Controller):
                     container_dict['memory'] = str(allocation['memory'])
                 container_dict.pop('resources')
 
-            container_dict['restart_policy'] = container_restart_policy
-            utils.check_for_restart_policy(container_dict)
-
-            if container_dict.get('volumeMounts'):
-                for volume in container_dict['volumeMounts']:
-                    volume['container_name'] = name
-                    container_volume_requests.append(volume)
-
             container_dict['status'] = consts.CREATING
             container_dict['interactive'] = True
             container_dict['capsule_id'] = new_capsule.id
-            new_container = objects.Container(context, **container_dict)
+            container_dict['restart_policy'] = container_restart_policy
+            if container_spec in init_containers_spec:
+                if capsule_restart_policy == "always":
+                    container_restart_policy = {"MaximumRetryCount": "10",
+                                                "Name": "on-failure"}
+                    container_dict['restart_policy'] = container_restart_policy
+                utils.check_for_restart_policy(container_dict)
+                new_container = objects.CapsuleInitContainer(context,
+                                                             **container_dict)
+            else:
+                utils.check_for_restart_policy(container_dict)
+                new_container = objects.CapsuleContainer(context,
+                                                         **container_dict)
             new_container.create(context)
-            new_capsule.containers_uuids.append(new_container.uuid)
+
+            if container_dict.get('volumeMounts'):
+                for volume in container_dict['volumeMounts']:
+                    volume['container_uuid'] = new_container.uuid
+                    container_volume_requests.append(volume)
 
         # Deal with the volume support
         requested_volumes = \
@@ -254,16 +275,23 @@ class CapsuleController(base.Controller):
         new_capsule.cpu = capsule_need_cpu
         new_capsule.memory = str(capsule_need_memory)
         new_capsule.save(context)
-        compute_api.capsule_create(context, new_capsule, requested_networks,
-                                   requested_volumes, extra_spec)
+
+        kwargs = {}
+        kwargs['extra_spec'] = extra_spec
+        kwargs['requested_networks'] = requested_networks
+        kwargs['requested_volumes'] = requested_volumes
+        kwargs['run'] = True
+        compute_api.container_create(context, new_capsule, **kwargs)
         # Set the HTTP Location Header
         pecan.response.location = link.build_url('capsules',
                                                  new_capsule.uuid)
 
         pecan.response.status = 202
         return view.format_capsule(pecan.request.host_url, new_capsule,
-                                   context)
+                                   context,
+                                   legacy_api_version=legacy_api_version)
 
+    @base.Controller.api_version("1.1", "1.31")
     @pecan.expose('json')
     @exception.wrap_pecan_controller_exception
     def get_one(self, capsule_ident):
@@ -271,10 +299,24 @@ class CapsuleController(base.Controller):
 
         :param capsule_ident: UUID or name of a capsule.
         """
+        return self._do_get_one(capsule_ident, legacy_api_version=True)
+
+    @base.Controller.api_version("1.32")  # noqa
+    @pecan.expose('json')
+    @exception.wrap_pecan_controller_exception
+    def get_one(self, capsule_ident):
+        """Retrieve information about the given capsule.
+
+        :param capsule_ident: UUID or name of a capsule.
+        """
+        return self._do_get_one(capsule_ident)
+
+    def _do_get_one(self, capsule_ident, legacy_api_version=False):
         context = pecan.request.context
         capsule = _get_capsule(capsule_ident)
         check_policy_on_capsule(capsule.as_dict(), "capsule:get")
-        return view.format_capsule(pecan.request.host_url, capsule, context)
+        return view.format_capsule(pecan.request.host_url, capsule, context,
+                                   legacy_api_version=legacy_api_version)
 
     @pecan.expose('json')
     @exception.wrap_pecan_controller_exception
@@ -293,44 +335,18 @@ class CapsuleController(base.Controller):
         compute_api = pecan.request.compute_api
         capsule.task_state = consts.CONTAINER_DELETING
         capsule.save(context)
-        compute_api.capsule_delete(context, capsule)
+        compute_api.container_stop(context, capsule, 10)
+        compute_api.container_delete(context, capsule)
         pecan.response.status = 204
 
     def _generate_name_for_capsule_container(self, new_capsule):
         """Generate a random name like: zeta-22-container."""
         name_gen = name_generator.NameGenerator()
         name = name_gen.generate()
-        if new_capsule.meta_name is None:
+        if new_capsule.name is None:
             return 'capsule-' + new_capsule.uuid + '-' + name
         else:
-            return 'capsule-' + new_capsule.meta_name + '-' + name
-
-    def _generate_name_for_capsule_sandbox(self, new_capsule):
-        """Generate sandbox name inside the capsule"""
-        if new_capsule.meta_name is None:
-            return 'capsule-' + new_capsule.uuid + '-' + 'sandbox'
-        else:
-            return 'capsule-' + new_capsule.meta_name + '-' + 'sandbox'
-
-    def _transfer_different_field(self, field_tpl,
-                                  field_container, **container_dict):
-        """Transfer the template specified field to container_field"""
-        if container_dict.get(field_tpl):
-            container_dict[field_container] = api_utils.string_or_none(
-                container_dict.get(field_tpl))
-            container_dict.pop(field_tpl)
-        return container_dict
-
-    def _transfer_list_to_str(self, container_dict, field):
-        if container_dict[field]:
-            dict = None
-            for k in range(0, len(container_dict[field])):
-                if dict:
-                    dict = dict + ' ' + container_dict[field][k]
-                else:
-                    dict = container_dict[field][k]
-            container_dict[field] = dict
-        return container_dict
+            return 'capsule-' + new_capsule.name + '-' + name
 
     def _build_requested_volumes(self, context, volume_spec,
                                  volume_mounts, capsule):
@@ -342,7 +358,7 @@ class CapsuleController(base.Controller):
         # volume.
         cinder_api = cinder.CinderAPI(context)
         volume_driver = "cinder"
-        requested_volumes = []
+        requested_volumes = {}
         volume_created = []
         try:
             for mount in volume_spec:
@@ -361,24 +377,31 @@ class CapsuleController(base.Controller):
                         auto_remove = True
 
                 mount_destination = None
-                container_name = None
+                container_uuid = None
+
+                volume_object = objects.Volume(
+                    context,
+                    cinder_volume_id=volume.id,
+                    volume_provider=volume_driver,
+                    user_id=context.user_id,
+                    project_id=context.project_id,
+                    auto_remove=auto_remove)
+                volume_object.create(context)
 
                 for item in volume_mounts:
                     if item['name'] == mount['name']:
                         mount_destination = item['mountPath']
-                        container_name = item['container_name']
-                        break
+                        container_uuid = item['container_uuid']
+                        volmapp = objects.VolumeMapping(
+                            context,
+                            container_path=mount_destination,
+                            user_id=context.user_id,
+                            project_id=context.project_id,
+                            volume_id=volume_object.id)
+                        requested_volumes.setdefault(container_uuid, [])
+                        requested_volumes[container_uuid].append(volmapp)
 
-                if mount_destination and container_name:
-                    volmapp = objects.VolumeMapping(
-                        context,
-                        volume_id=volume.id, volume_provider=volume_driver,
-                        container_path=mount_destination,
-                        user_id=context.user_id,
-                        project_id=context.project_id,
-                        auto_remove=auto_remove)
-                    requested_volumes.append({container_name: volmapp})
-                else:
+                if not mount_destination or not container_uuid:
                     msg = _("volume mount parameters is invalid.")
                     raise exception.Invalid(msg)
         except Exception as e:
