@@ -38,7 +38,9 @@ class KeystoneClientV3(object):
     def auth_url(self):
         # FIXME(pauloewerton): auth_url should be retrieved from keystone_auth
         # section by default
-        return CONF[ksconf.CFG_LEGACY_GROUP].auth_uri.replace('v2.0', 'v3')
+        url = CONF[ksconf.CFG_LEGACY_GROUP].www_authenticate_uri or \
+            CONF[ksconf.CFG_LEGACY_GROUP].auth_uri
+        return url.replace('v2.0', 'v3')
 
     @property
     def auth_token(self):
