@@ -16,16 +16,18 @@
 # https://docs.openstack.org/oslo.i18n/latest/user/usage.html
 
 """Utilities and helper functions."""
+import base64
+import binascii
 import eventlet
 import functools
 import inspect
-import json
 import mimetypes
 
 from oslo_concurrency import lockutils
 from oslo_concurrency import processutils
 from oslo_context import context as common_context
 from oslo_log import log as logging
+from oslo_serialization import jsonutils
 from oslo_utils import excutils
 from oslo_utils import strutils
 import pecan
@@ -34,6 +36,7 @@ import six
 from zun.api import utils as api_utils
 from zun.common import clients
 from zun.common import consts
+from zun.common.docker_image import reference as docker_image
 from zun.common import exception
 from zun.common.i18n import _
 from zun.common import privileged
@@ -159,19 +162,26 @@ def allow_all_content_types(f):
     return _do_allow_certain_content_types(f, mimetypes.types_map.values())
 
 
-def parse_image_name(image, driver=None):
-    image_parts = image.split(':', 1)
+def parse_image_name(image, driver=None, registry=None):
+    image_parts = docker_image.Reference.parse(image)
 
-    image_repo = image_parts[0]
+    image_repo = image_parts['name']
     if driver is None:
         driver = CONF.default_image_driver
     if driver == 'glance':
         image_tag = ''
-    else:
-        image_tag = 'latest'
+        return image_repo, image_tag
 
-    if len(image_parts) > 1:
-        image_tag = image_parts[1]
+    image_tag = 'latest'
+    if image_parts['tag']:
+        image_tag = image_parts['tag']
+
+    domain, _ = image_parts.split_hostname()
+    if not domain:
+        if registry:
+            image_repo = '%s/%s' % (registry.domain, image_repo)
+        elif CONF.docker.default_registry:
+            image_repo = '%s/%s' % (CONF.docker.default_registry, image_repo)
 
     return image_repo, image_tag
 
@@ -359,7 +369,7 @@ def check_capsule_template(tpl):
     tpl_json = tpl
     if isinstance(tpl, six.string_types):
         try:
-            tpl_json = json.loads(tpl)
+            tpl_json = jsonutils.loads(tpl)
         except Exception as e:
             raise exception.FailedParseStringToJson(e)
 
@@ -401,7 +411,22 @@ def capsule_get_container_spec(spec_field):
             container_spec[VALID_CONTAINER_FILED[key]] = \
                 container_spec.pop(key)
 
-    return containers_spec
+    init_containers_spec = spec_field.get('initContainers')
+    if init_containers_spec is not None:
+        for i in range(0, len(init_containers_spec)):
+            container_spec = init_containers_spec[i]
+            if 'image' not in container_spec.keys():
+                raise exception.InvalidCapsuleTemplate("Container "
+                                                       "image is needed")
+            # Remap the Capsule's container fields to native Zun
+            # container fields.
+            for key in list(container_spec.keys()):
+                container_spec[VALID_CONTAINER_FILED[key]] = \
+                    container_spec.pop(key)
+    else:
+        init_containers_spec = []
+
+    return containers_spec, init_containers_spec
 
 
 def capsule_get_volume_spec(spec_field):
@@ -466,6 +491,14 @@ def get_image(image_id):
     return image
 
 
+def get_registry(registry_id):
+    registry = api_utils.get_resource('Registry', registry_id)
+    if not registry:
+        raise exception.RegistryNotFound(registry=registry_id)
+
+    return registry
+
+
 def check_for_restart_policy(container_dict):
     """Check for restart policy input
 
@@ -487,6 +520,36 @@ def check_for_restart_policy(container_dict):
         container_dict.get('restart_policy')['MaximumRetryCount'] = '0'
 
 
+def build_exposed_ports(ports):
+
+    def validate_protocol(protocol):
+        if protocol not in ('tcp', 'udp'):
+            raise exception.InvalidValue(_(
+                "value %s is an invalid protocol") % protocol)
+
+    def validate_port(port):
+        try:
+            int(port)
+        except ValueError:
+            msg = _("value %s is invalid as publish port.") % port
+            raise exception.InvalidValue(msg)
+
+    exposed_ports = {}
+    for key, value in ports.items():
+        try:
+            port, protocol = key.split('/')
+        except ValueError:
+            port, protocol = key, 'tcp'
+
+        validate_protocol(protocol)
+        validate_port(port)
+
+        key = '/'.join([port, protocol])
+        exposed_ports[key] = value
+
+    return exposed_ports
+
+
 def build_requested_networks(context, nets):
     """Build requested networks by calling neutron client
 
@@ -630,3 +693,28 @@ def wrap_exception():
                     LOG.exception("Unexpected exception: %s", six.text_type(e))
         return decorated_function
     return helper
+
+
+def is_close(x, y, rel_tol=1e-06, abs_tol=0.0):
+    return abs(x - y) <= max(rel_tol * max(abs(x), abs(y)), abs_tol)
+
+
+def is_less_than(x, y):
+    if isinstance(x, int) and isinstance(y, int):
+        return x < y
+    if isinstance(x, float) or isinstance(y, float):
+        return False if (x - y) >= 0 or is_close(x, y) else True
+
+
+def encode_file_data(data):
+    if six.PY3 and isinstance(data, str):
+        data = data.encode('utf-8')
+    return base64.b64encode(data).decode('utf-8')
+
+
+def decode_file_data(data):
+    # Py3 raises binascii.Error instead of TypeError as in Py27
+    try:
+        return base64.b64decode(data)
+    except (TypeError, binascii.Error):
+        raise exception.Base64Exception()
