@@ -29,6 +29,7 @@ from sqlalchemy.sql.expression import desc
 from sqlalchemy.sql import func
 
 from zun.common import consts
+from zun.common import crypt
 from zun.common import exception
 from zun.common.i18n import _
 import zun.conf
@@ -129,7 +130,7 @@ class Connection(object):
 
         return query
 
-    def _add_filters(self, query, filters=None, filter_names=None):
+    def _add_filters(self, query, model, filters=None, filter_names=None):
         """Generic way to add filters to a Zun model"""
         if not filters:
             return query
@@ -141,10 +142,11 @@ class Connection(object):
             if name in filters:
                 value = filters[name]
                 if isinstance(value, list):
-                    column = getattr(models.Container, name)
+                    column = getattr(model, name)
                     query = query.filter(column.in_(value))
                 else:
-                    query = query.filter_by(**{name: value})
+                    column = getattr(model, name)
+                    query = query.filter(column == value)
 
         return query
 
@@ -153,13 +155,19 @@ class Connection(object):
                         'memory', 'host', 'task_state', 'status',
                         'auto_remove', 'uuid', 'capsule_id']
 
-        return self._add_filters(query, filters=filters,
+        return self._add_filters(query, models.Container, filters=filters,
                                  filter_names=filter_names)
 
-    def list_containers(self, context, filters=None, limit=None,
-                        marker=None, sort_key=None, sort_dir=None):
+    def _add_container_type_filter(self, container_type, query):
+        if container_type != consts.TYPE_ANY:
+            query = query.filter_by(container_type=container_type)
+        return query
+
+    def list_containers(self, context, container_type, filters=None,
+                        limit=None, marker=None, sort_key=None, sort_dir=None):
         query = model_query(models.Container)
         query = self._add_project_filters(context, query)
+        query = self._add_container_type_filter(container_type, query)
         query = self._add_containers_filters(query, filters)
         return _paginate_query(models.Container, limit, marker,
                                sort_key, sort_dir, query)
@@ -199,18 +207,20 @@ class Connection(object):
                                                    value=values['uuid'])
         return container
 
-    def get_container_by_uuid(self, context, container_uuid):
+    def get_container_by_uuid(self, context, container_type, container_uuid):
         query = model_query(models.Container)
         query = self._add_project_filters(context, query)
+        query = self._add_container_type_filter(container_type, query)
         query = query.filter_by(uuid=container_uuid)
         try:
             return query.one()
         except NoResultFound:
             raise exception.ContainerNotFound(container=container_uuid)
 
-    def get_container_by_name(self, context, container_name):
+    def get_container_by_name(self, context, container_type, container_name):
         query = model_query(models.Container)
         query = self._add_project_filters(context, query)
+        query = self._add_container_type_filter(container_type, query)
         query = query.filter_by(name=container_name)
         try:
             return query.one()
@@ -221,16 +231,17 @@ class Connection(object):
                                      'name. Please use the container uuid '
                                      'instead.')
 
-    def destroy_container(self, context, container_id):
+    def destroy_container(self, context, container_type, container_id):
         session = get_session()
         with session.begin():
             query = model_query(models.Container, session=session)
+            query = self._add_container_type_filter(container_type, query)
             query = add_identity_filter(query, container_id)
             count = query.delete()
             if count != 1:
                 raise exception.ContainerNotFound(container_id)
 
-    def update_container(self, context, container_id, values):
+    def update_container(self, context, container_type, container_id, values):
         # NOTE(dtantsur): this can lead to very strange errors
         if 'uuid' in values:
             msg = _("Cannot overwrite UUID for an existing Container.")
@@ -239,12 +250,13 @@ class Connection(object):
         if 'name' in values:
             self._validate_unique_container_name(context, values['name'])
 
-        return self._do_update_container(container_id, values)
+        return self._do_update_container(container_type, container_id, values)
 
-    def _do_update_container(self, container_id, values):
+    def _do_update_container(self, container_type, container_id, values):
         session = get_session()
         with session.begin():
             query = model_query(models.Container, session=session)
+            query = self._add_container_type_filter(container_type, query)
             query = add_identity_filter(query, container_id)
             try:
                 ref = query.with_lockmode('update').one()
@@ -255,19 +267,27 @@ class Connection(object):
         return ref
 
     def _add_volume_mappings_filters(self, query, filters):
-        filter_names = ['project_id', 'user_id', 'volume_id', 'container_path',
-                        'container_uuid', 'volume_provider']
-        return self._add_filters(query, filters=filters,
+        filter_names = ['project_id', 'user_id', 'volume_id',
+                        'container_path', 'container_uuid']
+        return self._add_filters(query, models.VolumeMapping, filters=filters,
                                  filter_names=filter_names)
 
     def list_volume_mappings(self, context, filters=None, limit=None,
                              marker=None, sort_key=None, sort_dir=None):
         query = model_query(models.VolumeMapping)
+        query = query.join(models.Volume)
         query = self._add_project_filters(context, query)
+        query = self._add_volume_filters(query, filters)
         query = self._add_volume_mappings_filters(query, filters)
         return _paginate_query(models.VolumeMapping, limit, marker,
                                sort_key, sort_dir, query)
 
+    def count_volume_mappings(self, context, **filters):
+        query = model_query(models.VolumeMapping)
+        query = self._add_project_filters(context, query)
+        query = self._add_volume_mappings_filters(query, filters)
+        return query.count()
+
     def create_volume_mapping(self, context, values):
         # ensure defaults are present for new volume_mappings
         if not values.get('uuid'):
@@ -322,6 +342,65 @@ class Connection(object):
             ref.update(values)
         return ref
 
+    def _add_volume_filters(self, query, filters):
+        filter_names = ['project_id', 'user_id', 'cinder_volume_id',
+                        'volume_provider']
+        return self._add_filters(query, models.Volume, filters=filters,
+                                 filter_names=filter_names)
+
+    def create_volume(self, context, values):
+        # ensure defaults are present for new volume_mappings
+        if not values.get('uuid'):
+            values['uuid'] = uuidutils.generate_uuid()
+
+        volume = models.Volume()
+        volume.update(values)
+        try:
+            volume.save()
+        except db_exc.DBDuplicateEntry:
+            raise exception.VolumeAlreadyExists(field='UUID',
+                                                value=values['uuid'])
+        return volume
+
+    def get_volume_by_id(self, context, volume_id):
+        query = model_query(models.Volume)
+        query = self._add_project_filters(context, query)
+        query = query.filter_by(id=volume_id)
+        try:
+            return query.one()
+        except NoResultFound:
+            raise exception.VolumeNotFound(volume_id)
+
+    def destroy_volume(self, context, volume_uuid):
+        session = get_session()
+        with session.begin():
+            query = model_query(models.Volume, session=session)
+            query = add_identity_filter(query, volume_uuid)
+            count = query.delete()
+            if count != 1:
+                raise exception.VolumeNotFound(volume_uuid)
+
+    def update_volume(self, context, volume_uuid, values):
+        # NOTE(dtantsur): this can lead to very strange errors
+        if 'uuid' in values:
+            msg = _("Cannot overwrite UUID for an existing Volume.")
+            raise exception.InvalidParameterValue(err=msg)
+
+        return self._do_update_volume(volume_uuid, values)
+
+    def _do_update_volume(self, volume_uuid, values):
+        session = get_session()
+        with session.begin():
+            query = model_query(models.Volume, session=session)
+            query = add_identity_filter(query, volume_uuid)
+            try:
+                ref = query.with_lockmode('update').one()
+            except NoResultFound:
+                raise exception.VolumeNotFound(volume_uuid)
+
+            ref.update(values)
+        return ref
+
     def destroy_zun_service(self, host, binary):
         session = get_session()
         with session.begin():
@@ -368,7 +447,7 @@ class Connection(object):
 
     def _add_zun_service_filters(self, query, filters):
         filter_names = ['disabled', 'host', 'binary', 'project_id', 'user_id']
-        return self._add_filters(query, filters=filters,
+        return self._add_filters(query, models.ZunService, filters=filters,
                                  filter_names=filter_names)
 
     def list_zun_services(self, filters=None, limit=None, marker=None,
@@ -429,7 +508,7 @@ class Connection(object):
 
     def _add_image_filters(self, query, filters):
         filter_names = ['repo', 'project_id', 'user_id', 'size']
-        return self._add_filters(query, filters=filters,
+        return self._add_filters(query, models.Image, filters=filters,
                                  filter_names=filter_names)
 
     def list_images(self, context, filters=None, limit=None, marker=None,
@@ -460,7 +539,8 @@ class Connection(object):
 
     def _add_resource_providers_filters(self, query, filters):
         filter_names = ['name', 'root_provider', 'parent_provider', 'can_host']
-        return self._add_filters(query, filters=filters,
+        return self._add_filters(query, models.ResourceProvider,
+                                 filters=filters,
                                  filter_names=filter_names)
 
     def list_resource_providers(self, context, filters=None, limit=None,
@@ -608,7 +688,7 @@ class Connection(object):
         filter_names = ['resource_provider_id', 'resource_class_id', 'total',
                         'reserved', 'min_unit', 'max_unit', 'step_size',
                         'allocation_ratio', 'is_nested']
-        return self._add_filters(query, filters=filters,
+        return self._add_filters(query, models.Inventory, filters=filters,
                                  filter_names=filter_names)
 
     def list_inventories(self, context, filters=None, limit=None,
@@ -668,7 +748,7 @@ class Connection(object):
     def _add_allocations_filters(self, query, filters):
         filter_names = ['resource_provider_id', 'resource_class_id',
                         'consumer_id', 'used', 'is_nested']
-        return self._add_filters(query, filters=filters,
+        return self._add_filters(query, models.Allocation, filters=filters,
                                  filter_names=filter_names)
 
     def list_allocations(self, context, filters=None, limit=None,
@@ -726,7 +806,7 @@ class Connection(object):
 
     def _add_compute_nodes_filters(self, query, filters):
         filter_names = ['hostname']
-        return self._add_filters(query, filters=filters,
+        return self._add_filters(query, models.ComputeNode, filters=filters,
                                  filter_names=filter_names)
 
     def list_compute_nodes(self, context, filters=None, limit=None,
@@ -803,85 +883,6 @@ class Connection(object):
             ref.update(values)
         return ref
 
-    def list_capsules(self, context, filters=None, limit=None,
-                      marker=None, sort_key=None, sort_dir=None):
-        query = model_query(models.Capsule)
-        query = self._add_project_filters(context, query)
-        query = self._add_capsules_filters(query, filters)
-        return _paginate_query(models.Capsule, limit, marker,
-                               sort_key, sort_dir, query)
-
-    def create_capsule(self, context, values):
-        # ensure defaults are present for new capsules
-        # here use the infra container uuid as the capsule uuid
-        if not values.get('uuid'):
-            values['uuid'] = uuidutils.generate_uuid()
-        capsule = models.Capsule()
-        capsule.update(values)
-        try:
-            capsule.save()
-        except db_exc.DBDuplicateEntry:
-            raise exception.CapsuleAlreadyExists(field='UUID',
-                                                 value=values['uuid'])
-        return capsule
-
-    def get_capsule_by_uuid(self, context, capsule_uuid):
-        query = model_query(models.Capsule)
-        query = self._add_project_filters(context, query)
-        query = query.filter_by(uuid=capsule_uuid)
-        try:
-            return query.one()
-        except NoResultFound:
-            raise exception.CapsuleNotFound(capsule=capsule_uuid)
-
-    def get_capsule_by_meta_name(self, context, capsule_name):
-        query = model_query(models.Capsule)
-        query = self._add_project_filters(context, query)
-        query = query.filter_by(meta_name=capsule_name)
-        try:
-            return query.one()
-        except NoResultFound:
-            raise exception.CapsuleNotFound(capsule=capsule_name)
-        except MultipleResultsFound:
-            raise exception.Conflict('Multiple capsules exist with same '
-                                     'name. Please use the capsule uuid '
-                                     'instead.')
-
-    def destroy_capsule(self, context, capsule_id):
-        session = get_session()
-        with session.begin():
-            query = model_query(models.Capsule, session=session)
-            query = add_identity_filter(query, capsule_id)
-            count = query.delete()
-            if count != 1:
-                raise exception.CapsuleNotFound(capsule_id)
-
-    def update_capsule(self, context, capsule_id, values):
-        if 'uuid' in values:
-            msg = _("Cannot overwrite UUID for an existing Capsule.")
-            raise exception.InvalidParameterValue(err=msg)
-
-        return self._do_update_capsule_id(capsule_id, values)
-
-    def _do_update_capsule_id(self, capsule_id, values):
-        session = get_session()
-        with session.begin():
-            query = model_query(models.Capsule, session=session)
-            query = add_identity_filter(query, capsule_id)
-            try:
-                ref = query.with_lockmode('update').one()
-            except NoResultFound:
-                raise exception.CapsuleNotFound(capsule=capsule_id)
-
-            ref.update(values)
-        return ref
-
-    def _add_capsules_filters(self, query, filters):
-        # filter_names = ['uuid', 'project_id', 'user_id', 'containers']
-        filter_names = ['uuid', 'project_id', 'user_id']
-        return self._add_filters(query, filters=filters,
-                                 filter_names=filter_names)
-
     def get_pci_device_by_addr(self, node_id, dev_addr):
         pci_dev_ref = model_query(models.PciDevice).\
             filter_by(compute_node_uuid=node_id).\
@@ -1053,8 +1054,8 @@ class Connection(object):
         try:
             quota_ref.save(session=session)
         except db_exc.DBDuplicateEntry:
-            raise exception.QuotaExists(project_id=project_id,
-                                        resource=resource)
+            raise exception.QuotaAlreadyExists(project_id=project_id,
+                                               resource=resource)
         return quota_ref
 
     def quota_get(self, context, project_id, resource):
@@ -1102,20 +1103,13 @@ class Connection(object):
     def quota_destroy_all_by_project(self, context, project_id):
         session = get_session()
         with session.begin():
-            quotas = model_query(context, models.Quota, session=session).\
-                filter_by(project_id).\
-                all()
-
-            for quota in quotas:
-                quota.delete(session=session)
-
-            quota_usages = model_query(context, models.QuotaUsage,
-                                       session=session).\
+            model_query(models.Quota, session=session).\
                 filter_by(project_id=project_id).\
-                all()
+                delete()
 
-            for quota_usage in quota_usages:
-                quota_usage.delete(session=session)
+            model_query(models.QuotaUsage, session=session).\
+                filter_by(project_id=project_id).\
+                delete()
 
     def quota_class_create(self, context, class_name, resource, limit):
         quota_class_ref = models.QuotaClass()
@@ -1177,7 +1171,7 @@ class Connection(object):
                 raise exception.QuotaClassNotFound(class_name=class_name)
 
     def quota_usage_get_all_by_project(self, context, project_id):
-        rows = model_query(context, models.QuotaUsage).\
+        rows = model_query(models.QuotaUsage).\
             filter_by(project_id=project_id).\
             all()
 
@@ -1188,6 +1182,19 @@ class Connection(object):
 
         return result
 
+    def _add_networks_filters(self, query, filters):
+        filter_names = ['name', 'neutron_net_id', 'project_id', 'user_id']
+        return self._add_filters(query, models.Network, filters=filters,
+                                 filter_names=filter_names)
+
+    def list_networks(self, context, filters=None, limit=None,
+                      marker=None, sort_key=None, sort_dir=None):
+        query = model_query(models.Network)
+        query = self._add_project_filters(context, query)
+        query = self._add_networks_filters(query, filters)
+        return _paginate_query(models.Network, limit, marker,
+                               sort_key, sort_dir, query)
+
     def create_network(self, context, values):
         # ensure defaults are present for new networks
         if not values.get('uuid'):
@@ -1235,6 +1242,15 @@ class Connection(object):
         except NoResultFound:
             raise exception.NetworkNotFound(network=network_uuid)
 
+    def destroy_network(self, context, network_uuid):
+        session = get_session()
+        with session.begin():
+            query = model_query(models.Network, session=session)
+            query = add_identity_filter(query, network_uuid)
+            count = query.delete()
+            if count != 1:
+                raise exception.NetworkNotFound(network=network_uuid)
+
     def list_exec_instances(self, context, filters=None, limit=None,
                             marker=None, sort_key=None, sort_dir=None):
         query = model_query(models.ExecInstance)
@@ -1244,7 +1260,7 @@ class Connection(object):
 
     def _add_exec_instances_filters(self, query, filters):
         filter_names = ['container_id', 'exec_id', 'token']
-        return self._add_filters(query, filters=filters,
+        return self._add_filters(query, models.ExecInstance, filters=filters,
                                  filter_names=filter_names)
 
     def create_exec_instance(self, context, values):
@@ -1256,3 +1272,140 @@ class Connection(object):
             raise exception.ExecInstanceAlreadyExists(
                 exec_id=values['exec_id'])
         return exec_inst
+
+    def count_usage(self, context, container_type, project_id, flag):
+        session = get_session()
+        if flag == 'containers':
+            project_query = session.query(
+                func.count(models.Container.id)). \
+                filter_by(project_id=project_id). \
+                filter_by(container_type=container_type)
+        elif flag in ['disk', 'cpu', 'memory']:
+            project_query = session.query(
+                func.sum(getattr(models.Container, flag))). \
+                filter_by(project_id=project_id). \
+                filter_by(container_type=container_type)
+
+        return project_query.first()
+
+    def _add_registries_filters(self, query, filters):
+        filter_names = ['name', 'domain', 'username', 'project_id', 'user_id']
+        return self._add_filters(query, models.Registry, filters=filters,
+                                 filter_names=filter_names)
+
+    def list_registries(self, context, filters=None, limit=None,
+                        marker=None, sort_key=None, sort_dir=None):
+        query = model_query(models.Registry)
+        query = self._add_project_filters(context, query)
+        query = self._add_registries_filters(query, filters)
+        result = _paginate_query(models.Registry, limit, marker,
+                                 sort_key, sort_dir, query)
+        for row in result:
+            row['password'] = crypt.decrypt(row['password'])
+        return result
+
+    def create_registry(self, context, values):
+        # ensure defaults are present for new registries
+        if not values.get('uuid'):
+            values['uuid'] = uuidutils.generate_uuid()
+
+        original_password = values.get('password')
+        if original_password:
+            values['password'] = crypt.encrypt(values.get('password'))
+
+        registry = models.Registry()
+        registry.update(values)
+        try:
+            registry.save()
+        except db_exc.DBDuplicateEntry:
+            raise exception.RegistryAlreadyExists(
+                field='UUID', value=values['uuid'])
+
+        if original_password:
+            # the password is encrypted but we want to return the original
+            # password
+            registry['password'] = original_password
+
+        return registry
+
+    def update_registry(self, context, registry_uuid, values):
+        # NOTE(dtantsur): this can lead to very strange errors
+        if 'uuid' in values:
+            msg = _("Cannot overwrite UUID for an existing registry.")
+            raise exception.InvalidParameterValue(err=msg)
+
+        original_password = values.get('password')
+        if original_password:
+            values['password'] = crypt.encrypt(values.get('password'))
+
+        updated = self._do_update_registry(registry_uuid, values)
+        if original_password:
+            # the password is encrypted but we want to return the original
+            # password
+            updated['password'] = original_password
+
+        return updated
+
+    def _do_update_registry(self, registry_uuid, values):
+        session = get_session()
+        with session.begin():
+            query = model_query(models.Registry, session=session)
+            query = add_identity_filter(query, registry_uuid)
+            try:
+                ref = query.with_lockmode('update').one()
+            except NoResultFound:
+                raise exception.RegistryNotFound(registry=registry_uuid)
+
+            ref.update(values)
+        return ref
+
+    def get_registry_by_id(self, context, registry_id):
+        query = model_query(models.Registry)
+        query = self._add_project_filters(context, query)
+        query = query.filter_by(id=registry_id)
+        try:
+            result = query.one()
+            result['password'] = crypt.decrypt(result['password'])
+            return result
+        except NoResultFound:
+            raise exception.RegistryNotFound(registry=registry_id)
+
+    def get_registry_by_uuid(self, context, registry_uuid):
+        query = model_query(models.Registry)
+        query = self._add_project_filters(context, query)
+        query = query.filter_by(uuid=registry_uuid)
+        try:
+            result = query.one()
+            result['password'] = crypt.decrypt(result['password'])
+            return result
+        except NoResultFound:
+            raise exception.RegistryNotFound(registry=registry_uuid)
+
+    def get_registry_by_name(self, context, registry_name):
+        query = model_query(models.Registry)
+        query = self._add_project_filters(context, query)
+        query = query.filter_by(name=registry_name)
+        try:
+            result = query.one()
+            result['password'] = crypt.decrypt(result['password'])
+            return result
+        except NoResultFound:
+            raise exception.RegistryNotFound(registry=registry_name)
+        except MultipleResultsFound:
+            raise exception.Conflict('Multiple registries exist with same '
+                                     'name. Please use the registry uuid '
+                                     'instead.')
+
+    def destroy_registry(self, context, registry_uuid):
+        session = get_session()
+        with session.begin():
+            query = model_query(models.Registry, session=session)
+            query = add_identity_filter(query, registry_uuid)
+            try:
+                count = query.delete()
+                if count != 1:
+                    raise exception.RegistryNotFound(registry=registry_uuid)
+            except db_exc.DBReferenceError:
+                raise exception.Conflict('Failed to delete registry '
+                                         '%(registry)s since it is in use.',
+                                         registry=registry_uuid)
