@@ -19,13 +19,17 @@ import six
 from oslo_log import log as logging
 from oslo_utils import excutils
 
+from zun.common.docker_image import reference as docker_image
 from zun.common import exception
 from zun.common.i18n import _
 from zun.common import utils
+import zun.conf
 from zun.container.docker import utils as docker_utils
 from zun.image import driver
 
 
+CONF = zun.conf.CONF
+
 LOG = logging.getLogger(__name__)
 
 
@@ -61,14 +65,31 @@ class DockerDriver(driver.ContainerImageDriver):
                 LOG.debug('Image %s not found locally', image)
                 return None
 
-    def _pull_image(self, repo, tag):
+    def _pull_image(self, repo, tag, registry):
+        auth_config = None
+        image_ref = docker_image.Reference.parse(repo)
+        registry_domain, remainder = image_ref.split_hostname()
+        if registry and registry.username:
+            auth_config = {'username': registry.username,
+                           'password': registry.password}
+        elif (registry_domain and
+                registry_domain == CONF.docker.default_registry and
+                CONF.docker.default_registry_username):
+            auth_config = {'username': CONF.docker.default_registry_username,
+                           'password': CONF.docker.default_registry_password}
+
         with docker_utils.docker_client() as docker:
             try:
-                docker.pull(repo, tag=tag)
+                docker.pull(repo, tag=tag, auth_config=auth_config)
             except errors.NotFound as e:
                 raise exception.ImageNotFound(message=six.text_type(e))
+            except errors.APIError as e:
+                LOG.exception('Error on pulling image')
+                message = _('Error on pulling image: %(repo)s:%(tag)s') % {
+                    'repo': repo, 'tag': tag}
+                raise exception.ZunException(message)
 
-    def pull_image(self, context, repo, tag, image_pull_policy):
+    def pull_image(self, context, repo, tag, image_pull_policy, registry):
         image_loaded = True
         image = self._search_image_on_host(repo, tag)
         if not utils.should_pull_image(image_pull_policy, bool(image)):
@@ -84,7 +105,7 @@ class DockerDriver(driver.ContainerImageDriver):
             LOG.debug('Pulling image from docker %(repo)s,'
                       ' context %(context)s',
                       {'repo': repo, 'context': context})
-            self._pull_image(repo, tag)
+            self._pull_image(repo, tag, registry)
             return {'image': repo, 'path': None}, image_loaded
         except exception.ImageNotFound:
             with excutils.save_and_reraise_exception():
@@ -98,10 +119,17 @@ class DockerDriver(driver.ContainerImageDriver):
             raise exception.ZunException(msg.format(e))
 
     def search_image(self, context, repo, tag, exact_match):
+        image_ref = docker_image.Reference.parse(repo)
+        registry, image_name = image_ref.split_hostname()
+        if registry and registry != 'docker.io':
+            # Images searching is only supported in DockerHub
+            msg = _('Image searching is not supported in registry: {0}')
+            raise exception.OperationNotSupported(msg.format(registry))
+
         with docker_utils.docker_client() as docker:
             try:
-                # TODO(hongbin): search image by both repo and tag
-                images = docker.search(repo)
+                # TODO(hongbin): search image by both name and tag
+                images = docker.search(image_name)
             except errors.APIError as api_error:
                 raise exception.ZunException(six.text_type(api_error))
             except Exception as e:
@@ -109,7 +137,7 @@ class DockerDriver(driver.ContainerImageDriver):
                 raise exception.ZunException(msg.format(e))
 
         if exact_match:
-            images = [i for i in images if i['name'] == repo]
+            images = [i for i in images if i['name'] == image_name]
 
         for image in images:
             image['metadata'] = {}
