@@ -19,12 +19,12 @@ from neutronclient.common import exceptions
 from oslo_log import log as logging
 from oslo_utils import excutils
 
-from zun.common import context as zun_context
 from zun.common import exception
 from zun.common.i18n import _
 import zun.conf
 from zun.network import network
 from zun.network import neutron
+from zun import objects
 from zun.objects import fields as obj_fields
 from zun.pci import manager as pci_manager
 from zun.pci import utils as pci_utils
@@ -36,6 +36,7 @@ LOG = logging.getLogger(__name__)
 
 BINDING_PROFILE = 'binding:profile'
 BINDING_HOST_ID = 'binding:host_id'
+DEVICE_OWNER = 'compute:kuryr'
 
 
 class KuryrNetwork(network.Network):
@@ -58,7 +59,7 @@ class KuryrNetwork(network.Network):
         will have both ipv4 and ipv6 addresses.
 
         What this method does is finding the subnets under the specified
-        neutron net, retrieving the cidr, gateway, subnetpool of each
+        neutron net, retrieving the cidr, gateway of each
         subnet, and compile the list of parameters for docker.create_network.
         """
         # find a v4 and/or v6 subnet of the network
@@ -89,8 +90,6 @@ class KuryrNetwork(network.Network):
         }
 
         if v4_subnet:
-            ipam_options["Options"]['neutron.pool.uuid'] = \
-                self._get_subnetpool(v4_subnet)
             ipam_options['Options']['neutron.subnet.uuid'] = \
                 v4_subnet.get('id')
             ipam_options["Config"].append({
@@ -98,11 +97,8 @@ class KuryrNetwork(network.Network):
                 "Gateway": v4_subnet['gateway_ip']
             })
 
-            options['neutron.pool.uuid'] = v4_subnet.get('subnetpool_id')
             options['neutron.subnet.uuid'] = v4_subnet.get('id')
         if v6_subnet:
-            ipam_options["Options"]['neutron.pool.v6.uuid'] = \
-                self._get_subnetpool(v6_subnet)
             ipam_options['Options']['neutron.subnet.v6.uuid'] = \
                 v6_subnet.get('id')
             ipam_options["Config"].append({
@@ -110,59 +106,79 @@ class KuryrNetwork(network.Network):
                 "Gateway": v6_subnet['gateway_ip']
             })
 
-            options['neutron.pool.v6.uuid'] = v6_subnet.get('subnetpool_id')
             options['neutron.subnet.v6.uuid'] = v6_subnet.get('id')
 
+        network_dict = {}
+        network_dict['project_id'] = self.context.project_id
+        network_dict['user_id'] = self.context.user_id
+        network_dict['name'] = name
+        network_dict['neutron_net_id'] = neutron_net_id
+        network = objects.Network(self.context, **network_dict)
+
+        for attempt in (1, 2, 3):
+            LOG.debug("Attempt (%s) to create network: %s", attempt, network)
+            created_network = self._create_network_attempt(
+                network, options, ipam_options)
+            if created_network:
+                return created_network
+            time.sleep(1)
+
+        raise exception.ZunException(_(
+            "Cannot create docker network after several attempts %s"))
+
+    def _create_network_attempt(self, network, options, ipam_options):
+        # The DB model has unique constraint on 'neutron_net_id' field
+        # which will guarantee only one request can create the network in here
+        # (and call docker.create_network later) if there are concurrent
+        # requests on creating networks for the same neutron net.
+        try:
+            network.create(self.context)
+        except exception.NetworkAlreadyExists as e:
+            if e.field != 'neutron_net_id':
+                raise
+
+            networks = objects.Network.list(
+                self.context,
+                filters={'neutron_net_id': network.neutron_net_id})
+            LOG.debug("network objects with 'neutron_net_id' as '%(net_id)s': "
+                      "%(networks)s",
+                      {"net_id": network.neutron_net_id,
+                       "networks": networks})
+            docker_networks = self.list_networks(names=[network.name])
+            LOG.debug("docker networks with name matching '%(name)s': "
+                      "%(networks)s",
+                      {"name": network.name,
+                       "networks": docker_networks})
+            if (networks and networks[0].network_id and
+                    docker_networks and
+                    networks[0].network_id == docker_networks[0]['Id']):
+                LOG.debug("Network (%s) has already been created in docker",
+                          network.name)
+                return networks[0]
+            else:
+                # Probably, there are concurrent requests on creating the
+                # network but the network is yet created in Docker.
+                # We return False and let the caller retry.
+                return False
+
         LOG.debug("Calling docker.create_network to create network %s, "
-                  "ipam_options %s, options %s", name, ipam_options, options)
-        docker_network = self.docker.create_network(
-            name=name,
-            driver=CONF.network.driver_name,
-            enable_ipv6=True if v6_subnet else False,
-            options=options,
-            ipam=ipam_options)
+                  "ipam_options %s, options %s",
+                  network.name, ipam_options, options)
+        enable_ipv6 = bool(options.get('neutron.subnet.v6.uuid'))
+        try:
+            docker_network = self.docker.create_network(
+                name=network.name,
+                driver=CONF.network.driver_name,
+                enable_ipv6=enable_ipv6,
+                options=options,
+                ipam=ipam_options)
+        except Exception:
+            with excutils.save_and_reraise_exception():
+                network.destroy()
 
-        return docker_network
-
-    def _check_valid_subnetpool(self, neutron_api,
-                                subnetpool_id, subnet_cidr):
-        """Check subnet's cidr matches with subnetpool prefixes or not"""
-        subnetpools = \
-            neutron_api.list_subnetpools(id=subnetpool_id)
-        subnetpools = subnetpools.get('subnetpools', [])
-        if not len(subnetpools):
-            return False
-        if subnet_cidr in subnetpools[0]['prefixes']:
-            return True
-        return False
-
-    def _get_subnetpool(self, subnet):
-        # NOTE(kiennt): Elevate admin privilege to list all subnetpools
-        #               across projects.
-        admin_context = zun_context.get_admin_context()
-        neutron_api = neutron.NeutronAPI(admin_context)
-        subnetpool_id = subnet.get('subnetpool_id')
-        if not subnetpool_id:
-            return None
-        if self._check_valid_subnetpool(neutron_api, subnetpool_id,
-                                        subnet['cidr']):
-            return subnetpool_id
-        # NOTE(kiennt): Subnetpool which was created by Kuryr-libnetwork
-        #               will be tagged with subnet_id.
-        kwargs = {
-            'tags': [subnet['id']],
-        }
-
-        subnetpools = \
-            neutron_api.list_subnetpools(**kwargs).get('subnetpools', [])
-        if not subnetpools:
-            return None
-        elif len(subnetpools) > 1:
-            raise exception.ZunException(_(
-                'Multiple Neutron subnetpools exist with prefixes %s') %
-                subnet['cidr'])
-        else:
-            return subnetpools[0]['id']
+        network.network_id = docker_network['Id']
+        network.save()
+        return network
 
     def _get_subnet(self, subnets, ip_version):
         subnets = [s for s in subnets if s['ip_version'] == ip_version]
@@ -175,8 +191,9 @@ class KuryrNetwork(network.Network):
                 "Multiple Neutron subnets exist with ip version %s") %
                 ip_version)
 
-    def remove_network(self, network_name):
-        self.docker.remove_network(network_name)
+    def remove_network(self, network):
+        self.docker.remove_network(network.name)
+        network.destroy()
 
     def inspect_network(self, network_name):
         return self.docker.inspect_network(network_name)
@@ -185,25 +202,18 @@ class KuryrNetwork(network.Network):
         return self.docker.networks(**kwargs)
 
     def create_or_update_port(self, container, network_name,
-                              requested_network, security_groups=None):
+                              requested_network, security_groups=None,
+                              set_binding_host=False):
         if requested_network.get('port'):
             neutron_port_id = requested_network.get('port')
             neutron_port = self.neutron_api.get_neutron_port(neutron_port_id)
-            # NOTE(hongbin): If existing port is specified, security_group_ids
-            # is ignored because existing port already has security groups.
-            # We might revisit this behaviour later. Alternatively, we could
-            # either throw an exception or overwrite the port's security
-            # groups.
-            if not container.security_groups:
-                container.security_groups = []
-            if neutron_port.get('security_groups'):
-                for sg in neutron_port['security_groups']:
-                    if sg not in container.security_groups:
-                        container.security_groups += [sg]
-
             # update device_id in port
             port_req_body = {'port': {'device_id': container.uuid}}
-            self.neutron_api.update_port(neutron_port_id, port_req_body)
+            if set_binding_host:
+                port_req_body['port']['device_owner'] = DEVICE_OWNER
+                port_req_body['port'][BINDING_HOST_ID] = container.host
+            self.neutron_api.update_port(neutron_port_id, port_req_body,
+                                         admin=True)
 
             # If there is pci_request_id, it should be a sriov port.
             # populate pci related info.
@@ -218,9 +228,8 @@ class KuryrNetwork(network.Network):
                 # NOTE(hongbin): Use admin context here because non-admin
                 # context might not be able to update some attributes
                 # (i.e. binding:profile).
-                admin_context = zun_context.get_admin_context()
-                neutron_api = neutron.NeutronAPI(admin_context)
-                neutron_api.update_port(neutron_port_id, port_req_body)
+                self.neutron_api.update_port(neutron_port_id, port_req_body,
+                                             admin=True)
         else:
             network = self.inspect_network(network_name)
             neutron_net_id = network['Options']['neutron.net.uuid']
@@ -229,12 +238,16 @@ class KuryrNetwork(network.Network):
                 'tenant_id': self.context.project_id,
                 'device_id': container.uuid,
             }
+            if set_binding_host:
+                port_dict['device_owner'] = DEVICE_OWNER
+                port_dict[BINDING_HOST_ID] = container.host
             ip_addr = requested_network.get("fixed_ip")
             if ip_addr:
                 port_dict['fixed_ips'] = [{'ip_address': ip_addr}]
             if security_groups is not None:
                 port_dict['security_groups'] = security_groups
-            neutron_port = self.neutron_api.create_port({'port': port_dict})
+            neutron_port = self.neutron_api.create_port({'port': port_dict},
+                                                        admin=True)
             neutron_port = neutron_port['port']
 
         preserve_on_delete = requested_network['preserve_on_delete']
@@ -268,9 +281,7 @@ class KuryrNetwork(network.Network):
         This method will create a neutron port, retrieve the ip address(es)
         of the port, and pass them to docker.connect_container_to_network.
         """
-        container_id = container.get_sandbox_id()
-        if not container_id:
-            container_id = container.container_id
+        container_id = container.container_id
 
         addresses, original_port = self.create_or_update_port(
             container, network_name, requested_network, security_groups)
@@ -308,9 +319,8 @@ class KuryrNetwork(network.Network):
 
             try:
                 # Requires admin creds to set port bindings
-                admin_context = zun_context.get_admin_context()
-                neutron_api = neutron.NeutronAPI(admin_context)
-                neutron_api.update_port(port_id, port_req_body)
+                self.neutron_api.update_port(port_id, port_req_body,
+                                             admin=True)
             except exception.PortNotFound:
                 LOG.debug('Unable to unbind port %s as it no longer '
                           'exists.', port_id)
@@ -326,9 +336,7 @@ class KuryrNetwork(network.Network):
 
     def disconnect_container_from_network(self, container, network_name,
                                           neutron_network_id=None):
-        container_id = container.get_sandbox_id()
-        if not container_id:
-            container_id = container.container_id
+        container_id = container.container_id
 
         addrs_list = []
         if container.addresses and neutron_network_id:
@@ -391,9 +399,8 @@ class KuryrNetwork(network.Network):
 
         try:
             # Requires admin creds to set port bindings
-            admin_context = zun_context.get_admin_context()
-            neutron_api = neutron.NeutronAPI(admin_context)
-            neutron_api.update_port(port_id, port_req_body)
+            self.neutron_api.update_port(port_id, port_req_body,
+                                         admin=True)
         except exception.PortNotFound:
             LOG.debug('Unable to unbind port %s as it no longer '
                       'exists.', port_id)
@@ -402,10 +409,6 @@ class KuryrNetwork(network.Network):
                           port_id)
 
     def add_security_groups_to_ports(self, container, security_group_ids):
-        container_id = container.get_sandbox_id()
-        if not container_id:
-            container_id = container.container_id
-
         port_ids = set()
         for addrs_list in container.addresses.values():
             for addr in addrs_list:
@@ -426,10 +429,9 @@ class KuryrNetwork(network.Network):
                          "to port %(port_id)s",
                          {'security_group_ids': security_group_ids,
                           'port_id': port['id']})
-                admin_context = zun_context.get_admin_context()
-                neutron_api = neutron.NeutronAPI(admin_context)
-                neutron_api.update_port(port['id'],
-                                        {'port': updated_port})
+                self.neutron_api.update_port(port['id'],
+                                             {'port': updated_port},
+                                             admin=True)
             except exceptions.NeutronClientException as e:
                 exc_info = sys.exc_info()
                 if e.status_code == 400:
@@ -442,10 +444,6 @@ class KuryrNetwork(network.Network):
                     LOG.exception("Neutron Error:")
 
     def remove_security_groups_from_ports(self, container, security_group_ids):
-        container_id = container.get_sandbox_id()
-        if not container_id:
-            container_id = container.container_id
-
         port_ids = set()
         for addrs_list in container.addresses.values():
             for addr in addrs_list:
@@ -464,10 +462,9 @@ class KuryrNetwork(network.Network):
                          "from port %(port_id)s",
                          {'security_group_ids': security_group_ids,
                           'port_id': port['id']})
-                admin_context = zun_context.get_admin_context()
-                neutron_api = neutron.NeutronAPI(admin_context)
-                neutron_api.update_port(port['id'],
-                                        {'port': updated_port})
+                self.neutron_api.update_port(port['id'],
+                                             {'port': updated_port},
+                                             admin=True)
             except exceptions.NeutronClientException as e:
                 exc_info = sys.exc_info()
                 if e.status_code == 400:
