@@ -17,17 +17,24 @@ from webtest.app import AppError
 from oslo_utils import uuidutils
 
 from zun.common import exception
+import zun.conf
 from zun import objects
 from zun.tests.unit.api import base as api_base
 from zun.tests.unit.db import utils
 
 
-class TestImageController(api_base.FunctionalTest):
-    @patch('zun.compute.api.API.image_pull')
-    def test_image_pull(self, mock_image_pull):
-        mock_image_pull.side_effect = lambda x, y: y
+CONF = zun.conf.CONF
 
-        params = ('{"repo": "hello-world"}')
+
+class TestImageController(api_base.FunctionalTest):
+    @mock.patch('zun.common.policy.enforce', return_value=True)
+    @patch('zun.compute.api.API.image_pull')
+    @patch('zun.objects.ComputeNode.get_by_name')
+    def test_image_pull(self, mock_get, mock_image_pull, mock_policy_enforce):
+        mock_image_pull.side_effect = lambda x, y: y
+        mock_get.return_value = mock.Mock(hostname='fake-host')
+
+        params = ('{"repo": "hello-world", "host": "fake-host"}')
         response = self.post('/v1/images/',
                              params=params,
                              content_type='application/json')
@@ -35,7 +42,7 @@ class TestImageController(api_base.FunctionalTest):
         self.assertEqual(202, response.status_int)
         self.assertTrue(mock_image_pull.called)
 
-        params = ('{"repo": "hello-world:test"}')
+        params = ('{"repo": "hello-world:test", "host": "fake-host"}')
         response = self.post('/v1/images/',
                              params=params,
                              content_type='application/json')
@@ -53,11 +60,15 @@ class TestImageController(api_base.FunctionalTest):
                       content_type='application/json')
         self.assertTrue(mock_image_pull.not_called)
 
+    @mock.patch('zun.common.policy.enforce', return_value=True)
     @patch('zun.compute.api.API.image_pull')
-    def test_image_pull_conflict(self, mock_image_pull):
+    @patch('zun.objects.ComputeNode.get_by_name')
+    def test_image_pull_conflict(
+            self, mock_get, mock_image_pull, mock_policy_enforce):
         mock_image_pull.side_effect = lambda x, y: y
+        mock_get.return_value = mock.Mock(hostname='fake-host')
 
-        params = ('{"repo": "hello-world"}')
+        params = ('{"repo": "hello-world", "host": "fake-host"}')
         response = self.post('/v1/images/',
                              params=params,
                              content_type='application/json')
@@ -68,25 +79,32 @@ class TestImageController(api_base.FunctionalTest):
                           params=params, content_type='application/json')
         self.assertTrue(mock_image_pull.not_called)
 
+    @mock.patch('zun.common.policy.enforce', return_value=True)
     @patch('zun.compute.api.API.image_pull')
+    @patch('zun.objects.ComputeNode.get_by_name')
     def test_pull_image_set_project_id_and_user_id(
-            self, mock_image_pull):
+            self, mock_get, mock_image_pull, mock_policy_enforce):
         def _create_side_effect(cnxt, image):
             self.assertEqual(self.context.project_id, image.project_id)
             self.assertEqual(self.context.user_id, image.user_id)
             return image
         mock_image_pull.side_effect = _create_side_effect
+        mock_get.return_value = mock.Mock(hostname='fake-host')
 
-        params = ('{"repo": "hello-world"}')
+        params = ('{"repo": "hello-world", "host": "foo"}')
         self.post('/v1/images/',
                   params=params,
                   content_type='application/json')
 
+    @mock.patch('zun.common.policy.enforce', return_value=True)
     @patch('zun.compute.api.API.image_pull')
-    def test_image_pull_with_tag(self, mock_image_pull):
+    @patch('zun.objects.ComputeNode.get_by_name')
+    def test_image_pull_with_tag(
+            self, mock_get, mock_image_pull, mock_policy_enforce):
         mock_image_pull.side_effect = lambda x, y: y
+        mock_get.return_value = mock.Mock(hostname='fake-host')
 
-        params = ('{"repo": "hello-world:latest"}')
+        params = ('{"repo": "hello-world:latest", "host": "fake-host"}')
         response = self.post('/v1/images/',
                              params=params,
                              content_type='application/json')
@@ -94,8 +112,9 @@ class TestImageController(api_base.FunctionalTest):
         self.assertEqual(202, response.status_int)
         self.assertTrue(mock_image_pull.called)
 
+    @mock.patch('zun.common.policy.enforce', return_value=True)
     @patch('zun.objects.Image.list')
-    def test_get_all_images(self, mock_image_list):
+    def test_get_all_images(self, mock_image_list, mock_policy_enforce):
         test_image = utils.get_test_image()
         images = [objects.Image(self.context, **test_image)]
         mock_image_list.return_value = images
@@ -110,6 +129,8 @@ class TestImageController(api_base.FunctionalTest):
         self.assertEqual(1, len(actual_images))
         self.assertEqual(test_image['uuid'],
                          actual_images[0].get('uuid'))
+        self.assertEqual(test_image['host'],
+                         actual_images[0].get('host'))
 
     @patch('zun.common.policy.enforce')
     @patch('zun.objects.Image.get_by_uuid')
@@ -126,10 +147,13 @@ class TestImageController(api_base.FunctionalTest):
         self.assertEqual(200, response.status_int)
         self.assertEqual(test_image['uuid'],
                          response.json['uuid'])
+        self.assertEqual(test_image['host'],
+                         response.json['host'])
 
+    @mock.patch('zun.common.policy.enforce', return_value=True)
     @patch('zun.objects.Image.list')
-    def test_get_all_images_with_pagination_marker(self, mock_image_list
-                                                   ):
+    def test_get_all_images_with_pagination_marker(
+            self, mock_image_list, mock_policy_enforce):
         image_list = []
         for id_ in range(4):
             test_image = utils.create_test_image(
@@ -154,7 +178,7 @@ class TestImageController(api_base.FunctionalTest):
         response = self.get('/v1/images/redis/search/')
         self.assertEqual(200, response.status_int)
         mock_image_search.assert_called_once_with(
-            mock.ANY, 'redis', None, False)
+            mock.ANY, 'redis', CONF.default_image_driver, False)
 
     @patch('zun.compute.api.API.image_search')
     def test_search_image_with_tag(self, mock_image_search):
@@ -162,14 +186,14 @@ class TestImageController(api_base.FunctionalTest):
         response = self.get('/v1/images/redis:test/search/')
         self.assertEqual(200, response.status_int)
         mock_image_search.assert_called_once_with(
-            mock.ANY, 'redis:test', None, False)
+            mock.ANY, 'redis:test', CONF.default_image_driver, False)
 
     @patch('zun.compute.api.API.image_search')
     def test_search_image_not_found(self, mock_image_search):
         mock_image_search.side_effect = exception.ImageNotFound
         self.assertRaises(AppError, self.get, '/v1/images/redis/search/')
         mock_image_search.assert_called_once_with(
-            mock.ANY, 'redis', None, False)
+            mock.ANY, 'redis', CONF.default_image_driver, False)
 
     @patch('zun.compute.rpcapi.API.image_search')
     def test_search_image_with_exact_match_true(self, mock_image_search):
@@ -221,7 +245,7 @@ class TestImageEnforcement(api_base.FunctionalTest):
             expect_errors=True)
 
     def test_policy_disallow_create(self):
-        params = ('{"repo": "foo"}')
+        params = ('{"repo": "foo", "host": "foo"}')
         self._common_policy_check(
             'image:pull', self.post, '/v1/images/',
             params=params,
