@@ -14,6 +14,7 @@
 
 import mock
 
+from oslo_utils import uuidutils
 from six import StringIO
 
 from zun.common import consts
@@ -21,6 +22,7 @@ from zun.common import exception
 from zun.compute import claims
 from zun.compute import manager
 import zun.conf
+from zun import objects
 from zun.objects.container import Container
 from zun.objects.container_action import ContainerActionEvent
 from zun.objects.exec_instance import ExecInstance
@@ -50,14 +52,21 @@ class FakeResourceTracker(object):
 
 class FakeVolumeMapping(object):
 
-    volume_provider = 'fake_provider'
-    container_path = 'fake_path'
-    container_uuid = 'fake-cid'
-    volume_id = 'fake-vid'
-    auto_remove = False
-
     def __init__(self):
         self.__class__.volumes = []
+        self.volume_provider = 'fake_provider'
+        self.container_path = 'fake_path'
+        self.container_uuid = 'fake-cid'
+        self.cinder_volume_id = 'fake-vid'
+        self.volume_id = 123
+        self.connection_info = None
+        self.auto_remove = False
+        self.uuid = 'fake-uuid'
+        self.volume = mock.Mock()
+        self.volume.refresh.side_effect = self._volume_refresh
+
+    def _volume_refresh(self):
+        self.connection_info = 'refresh-info'
 
     def create(self, context):
         self.__class__.volumes.append(self)
@@ -65,14 +74,21 @@ class FakeVolumeMapping(object):
     def destroy(self):
         self.__class__.volumes.remove(self)
 
+    def obj_attr_is_set(self, name):
+        return bool(self.__class__.volumes)
+
     @classmethod
     def list_by_container(cls, context, container_id):
         return cls.volumes
 
     @classmethod
-    def list_by_volume(cls, context, volume_id):
+    def list_by_cinder_volume(cls, context, volume_id):
         return cls.volumes
 
+    @classmethod
+    def count(cls, context, **filters):
+        return len(cls.volumes)
+
 
 class TestManager(base.TestCase):
 
@@ -125,18 +141,17 @@ class TestManager(base.TestCase):
         mock_container_start.assert_called_once_with(self.context,
                                                      container)
 
-    @mock.patch.object(manager.Manager, 'container_reboot')
+    @mock.patch.object(manager.Manager, 'container_start')
     @mock.patch.object(Container, 'save')
     def test_container_reboot_after_host_reboot(self, mock_save,
-                                                mock_container_reboot):
+                                                mock_container_start):
         container_1 = Container(self.context, **utils.get_test_container())
         container_1.status = consts.RUNNING
         self.compute_manager.restore_running_container(self.context,
                                                        container_1,
                                                        consts.STOPPED)
-        mock_container_reboot.assert_called_once_with(self.context,
-                                                      container_1,
-                                                      10)
+        mock_container_start.assert_called_once_with(self.context,
+                                                     container_1)
 
     @mock.patch.object(manager.Manager, 'container_stop')
     @mock.patch.object(Container, 'save')
@@ -195,7 +210,7 @@ class TestManager(base.TestCase):
                                                   networks, volumes)
         mock_save.assert_called_with(self.context)
         mock_pull.assert_any_call(self.context, container.image, '',
-                                  'always', 'glance')
+                                  'always', 'glance', registry=None)
         mock_create.assert_called_once_with(self.context, container, image,
                                             networks, volumes)
         mock_event_start.assert_called_once()
@@ -317,29 +332,31 @@ class TestManager(base.TestCase):
     @mock.patch.object(ContainerActionEvent, 'event_finish')
     @mock.patch('zun.common.utils.spawn_n')
     @mock.patch.object(Container, 'save')
+    @mock.patch.object(VolumeMapping, 'count',
+                       side_effect=FakeVolumeMapping.count)
     @mock.patch.object(VolumeMapping, 'list_by_container',
                        side_effect=FakeVolumeMapping.list_by_container)
     @mock.patch.object(fake_driver, 'pull_image')
     @mock.patch.object(fake_driver, 'detach_volume')
     @mock.patch.object(fake_driver, 'attach_volume')
-    @mock.patch.object(fake_driver, 'get_volume_status')
+    @mock.patch.object(fake_driver, 'is_volume_available')
     @mock.patch.object(fake_driver, 'create')
     @mock.patch.object(fake_driver, 'start')
     def test_container_run(
             self, mock_start, mock_create,
-            mock_get_volume_status, mock_attach_volume,
-            mock_detach_volume, mock_pull, mock_list_by_container, mock_save,
-            mock_spawn_n, mock_event_finish, mock_event_start):
+            mock_is_volume_available, mock_attach_volume,
+            mock_detach_volume, mock_pull, mock_list_by_container, mock_count,
+            mock_save, mock_spawn_n, mock_event_finish, mock_event_start):
         container = Container(self.context, **utils.get_test_container())
         image = {'image': 'repo', 'path': 'out_path', 'driver': 'glance'}
         mock_create.return_value = container
         mock_pull.return_value = image, False
-        mock_get_volume_status.return_value = 'available'
+        mock_is_volume_available.return_value = True, False
         mock_spawn_n.side_effect = lambda f, *x, **y: f(*x, **y)
         container.status = 'Stopped'
         self.compute_manager._resource_tracker = FakeResourceTracker()
         networks = []
-        volumes = [FakeVolumeMapping()]
+        volumes = {container.uuid: [FakeVolumeMapping()]}
         self.compute_manager.container_create(
             self.context,
             requested_networks=networks,
@@ -348,13 +365,13 @@ class TestManager(base.TestCase):
             limits=None, run=True)
         mock_save.assert_called_with(self.context)
         mock_pull.assert_any_call(self.context, container.image, '',
-                                  'always', 'glance')
+                                  'always', 'glance', registry=None)
         mock_create.assert_called_once_with(self.context, container, image,
                                             networks, volumes)
         mock_start.assert_called_once_with(self.context, container)
         mock_attach_volume.assert_called_once()
         mock_detach_volume.assert_not_called()
-        mock_get_volume_status.assert_called_once()
+        mock_is_volume_available.assert_called_once()
         self.assertEqual(1, len(FakeVolumeMapping.volumes))
 
     @mock.patch.object(fake_driver, 'delete_volume')
@@ -362,24 +379,26 @@ class TestManager(base.TestCase):
     @mock.patch.object(ContainerActionEvent, 'event_finish')
     @mock.patch('zun.common.utils.spawn_n')
     @mock.patch.object(Container, 'save')
-    @mock.patch.object(VolumeMapping, 'list_by_volume',
-                       side_effect=FakeVolumeMapping.list_by_volume)
+    @mock.patch.object(VolumeMapping, 'count',
+                       side_effect=FakeVolumeMapping.count)
+    @mock.patch.object(VolumeMapping, 'list_by_cinder_volume',
+                       side_effect=FakeVolumeMapping.list_by_cinder_volume)
     @mock.patch.object(VolumeMapping, 'list_by_container',
                        side_effect=FakeVolumeMapping.list_by_container)
     @mock.patch.object(fake_driver, 'pull_image')
     @mock.patch.object(fake_driver, 'detach_volume')
     @mock.patch.object(fake_driver, 'attach_volume')
-    @mock.patch.object(fake_driver, 'get_volume_status')
+    @mock.patch.object(fake_driver, 'is_volume_available')
     @mock.patch.object(fake_driver, 'create')
     @mock.patch.object(fake_driver, 'start')
     def test_container_run_driver_attach_failed(
             self, mock_start, mock_create,
-            mock_get_volume_status, mock_attach_volume,
+            mock_is_volume_available, mock_attach_volume,
             mock_detach_volume, mock_pull, mock_list_by_container,
-            mock_list_by_volume, mock_save,
+            mock_list_by_volume, mock_count, mock_save,
             mock_spawn_n, mock_event_finish, mock_event_start,
             mock_delete_volume):
-        mock_get_volume_status.return_value = 'available'
+        mock_is_volume_available.return_value = True, False
         mock_attach_volume.side_effect = [None, base.TestingException("fake")]
         container = Container(self.context, **utils.get_test_container())
         vol = FakeVolumeMapping()
@@ -393,7 +412,7 @@ class TestManager(base.TestCase):
         container.status = 'Stopped'
         self.compute_manager._resource_tracker = FakeResourceTracker()
         networks = []
-        volumes = [vol, vol2]
+        volumes = {container.uuid: [vol, vol2]}
         self.assertRaises(
             base.TestingException,
             self.compute_manager.container_create,
@@ -416,30 +435,32 @@ class TestManager(base.TestCase):
     @mock.patch.object(ContainerActionEvent, 'event_finish')
     @mock.patch('zun.common.utils.spawn_n')
     @mock.patch.object(Container, 'save')
-    @mock.patch.object(VolumeMapping, 'list_by_volume',
-                       side_effect=FakeVolumeMapping.list_by_volume)
+    @mock.patch.object(VolumeMapping, 'count',
+                       side_effect=FakeVolumeMapping.count)
+    @mock.patch.object(VolumeMapping, 'list_by_cinder_volume',
+                       side_effect=FakeVolumeMapping.list_by_cinder_volume)
     @mock.patch.object(VolumeMapping, 'list_by_container',
                        side_effect=FakeVolumeMapping.list_by_container)
     @mock.patch.object(fake_driver, 'detach_volume')
     @mock.patch.object(fake_driver, 'attach_volume')
-    @mock.patch.object(fake_driver, 'get_volume_status')
+    @mock.patch.object(fake_driver, 'is_volume_available')
     @mock.patch.object(fake_driver, 'pull_image')
     def test_container_run_image_not_found(
-            self, mock_pull, mock_get_volume_status,
+            self, mock_pull, mock_is_volume_available,
             mock_attach_volume, mock_detach_volume,
-            mock_list_by_container, mock_list_by_volume,
+            mock_list_by_container, mock_list_by_volume, mock_count,
             mock_save, mock_spawn_n, mock_event_finish,
             mock_event_start):
         container_dict = utils.get_test_container(
             image='test:latest', image_driver='docker',
             image_pull_policy='ifnotpresent')
         container = Container(self.context, **container_dict)
-        mock_get_volume_status.return_value = 'available'
+        mock_is_volume_available.return_value = True, False
         mock_pull.side_effect = exception.ImageNotFound(
             message="Image Not Found")
         mock_spawn_n.side_effect = lambda f, *x, **y: f(*x, **y)
         networks = []
-        volumes = [FakeVolumeMapping()]
+        volumes = {container.uuid: [FakeVolumeMapping()]}
         self.assertRaises(
             exception.ImageNotFound,
             self.compute_manager.container_create,
@@ -452,40 +473,43 @@ class TestManager(base.TestCase):
         self.assertEqual('Error', container.status)
         self.assertEqual('Image Not Found', container.status_reason)
         mock_pull.assert_called_once_with(self.context, 'test', 'latest',
-                                          'ifnotpresent', 'docker')
+                                          'ifnotpresent', 'docker',
+                                          registry=None)
         mock_attach_volume.assert_called_once()
         mock_detach_volume.assert_called_once()
-        mock_get_volume_status.assert_called_once()
+        mock_is_volume_available.assert_called_once()
         self.assertEqual(0, len(FakeVolumeMapping.volumes))
 
     @mock.patch.object(ContainerActionEvent, 'event_start')
     @mock.patch.object(ContainerActionEvent, 'event_finish')
     @mock.patch('zun.common.utils.spawn_n')
     @mock.patch.object(Container, 'save')
-    @mock.patch.object(VolumeMapping, 'list_by_volume',
-                       side_effect=FakeVolumeMapping.list_by_volume)
+    @mock.patch.object(VolumeMapping, 'count',
+                       side_effect=FakeVolumeMapping.count)
+    @mock.patch.object(VolumeMapping, 'list_by_cinder_volume',
+                       side_effect=FakeVolumeMapping.list_by_cinder_volume)
     @mock.patch.object(VolumeMapping, 'list_by_container',
                        side_effect=FakeVolumeMapping.list_by_container)
     @mock.patch.object(fake_driver, 'detach_volume')
     @mock.patch.object(fake_driver, 'attach_volume')
-    @mock.patch.object(fake_driver, 'get_volume_status')
+    @mock.patch.object(fake_driver, 'is_volume_available')
     @mock.patch.object(fake_driver, 'pull_image')
     def test_container_run_image_pull_exception_raised(
-            self, mock_pull, mock_get_volume_status,
+            self, mock_pull, mock_is_volume_available,
             mock_attach_volume, mock_detach_volume,
-            mock_list_by_container, mock_list_by_volume,
+            mock_list_by_container, mock_list_by_volume, mock_count,
             mock_save, mock_spawn_n, mock_event_finish,
             mock_event_start):
         container_dict = utils.get_test_container(
             image='test:latest', image_driver='docker',
             image_pull_policy='ifnotpresent')
         container = Container(self.context, **container_dict)
-        mock_get_volume_status.return_value = 'available'
+        mock_is_volume_available.return_value = True, False
         mock_pull.side_effect = exception.ZunException(
             message="Image Not Found")
         mock_spawn_n.side_effect = lambda f, *x, **y: f(*x, **y)
         networks = []
-        volumes = [FakeVolumeMapping()]
+        volumes = {container.uuid: [FakeVolumeMapping()]}
         self.assertRaises(
             exception.ZunException,
             self.compute_manager.container_create,
@@ -498,40 +522,43 @@ class TestManager(base.TestCase):
         self.assertEqual('Error', container.status)
         self.assertEqual('Image Not Found', container.status_reason)
         mock_pull.assert_called_once_with(self.context, 'test', 'latest',
-                                          'ifnotpresent', 'docker')
+                                          'ifnotpresent', 'docker',
+                                          registry=None)
         mock_attach_volume.assert_called_once()
         mock_detach_volume.assert_called_once()
-        mock_get_volume_status.assert_called_once()
+        mock_is_volume_available.assert_called_once()
         self.assertEqual(0, len(FakeVolumeMapping.volumes))
 
     @mock.patch.object(ContainerActionEvent, 'event_start')
     @mock.patch.object(ContainerActionEvent, 'event_finish')
     @mock.patch('zun.common.utils.spawn_n')
     @mock.patch.object(Container, 'save')
-    @mock.patch.object(VolumeMapping, 'list_by_volume',
-                       side_effect=FakeVolumeMapping.list_by_volume)
+    @mock.patch.object(VolumeMapping, 'count',
+                       side_effect=FakeVolumeMapping.count)
+    @mock.patch.object(VolumeMapping, 'list_by_cinder_volume',
+                       side_effect=FakeVolumeMapping.list_by_cinder_volume)
     @mock.patch.object(VolumeMapping, 'list_by_container',
                        side_effect=FakeVolumeMapping.list_by_container)
     @mock.patch.object(fake_driver, 'detach_volume')
     @mock.patch.object(fake_driver, 'attach_volume')
-    @mock.patch.object(fake_driver, 'get_volume_status')
+    @mock.patch.object(fake_driver, 'is_volume_available')
     @mock.patch.object(fake_driver, 'pull_image')
     def test_container_run_image_pull_docker_error(
-            self, mock_pull, mock_get_volume_status,
+            self, mock_pull, mock_is_volume_available,
             mock_attach_volume, mock_detach_volume,
-            mock_list_by_container, mock_list_by_volume,
+            mock_list_by_container, mock_list_by_volume, mock_count,
             mock_save, mock_spawn_n, mock_event_finish,
             mock_event_start):
         container_dict = utils.get_test_container(
             image='test:latest', image_driver='docker',
             image_pull_policy='ifnotpresent')
         container = Container(self.context, **container_dict)
-        mock_get_volume_status.return_value = 'available'
+        mock_is_volume_available.return_value = True, False
         mock_pull.side_effect = exception.DockerError(
             message="Docker Error occurred")
         mock_spawn_n.side_effect = lambda f, *x, **y: f(*x, **y)
         networks = []
-        volumes = [FakeVolumeMapping()]
+        volumes = {container.uuid: [FakeVolumeMapping()]}
         self.assertRaises(
             exception.DockerError,
             self.compute_manager.container_create,
@@ -544,42 +571,45 @@ class TestManager(base.TestCase):
         self.assertEqual('Error', container.status)
         self.assertEqual('Docker Error occurred', container.status_reason)
         mock_pull.assert_called_once_with(self.context, 'test', 'latest',
-                                          'ifnotpresent', 'docker')
+                                          'ifnotpresent', 'docker',
+                                          registry=None)
         mock_attach_volume.assert_called_once()
         mock_detach_volume.assert_called_once()
-        mock_get_volume_status.assert_called_once()
+        mock_is_volume_available.assert_called_once()
         self.assertEqual(0, len(FakeVolumeMapping.volumes))
 
     @mock.patch.object(ContainerActionEvent, 'event_start')
     @mock.patch.object(ContainerActionEvent, 'event_finish')
     @mock.patch('zun.common.utils.spawn_n')
     @mock.patch.object(Container, 'save')
-    @mock.patch.object(VolumeMapping, 'list_by_volume',
-                       side_effect=FakeVolumeMapping.list_by_volume)
+    @mock.patch.object(VolumeMapping, 'count',
+                       side_effect=FakeVolumeMapping.count)
+    @mock.patch.object(VolumeMapping, 'list_by_cinder_volume',
+                       side_effect=FakeVolumeMapping.list_by_cinder_volume)
     @mock.patch.object(VolumeMapping, 'list_by_container',
                        side_effect=FakeVolumeMapping.list_by_container)
     @mock.patch.object(fake_driver, 'detach_volume')
     @mock.patch.object(fake_driver, 'attach_volume')
-    @mock.patch.object(fake_driver, 'get_volume_status')
+    @mock.patch.object(fake_driver, 'is_volume_available')
     @mock.patch.object(fake_driver, 'pull_image')
     @mock.patch.object(fake_driver, 'create')
     def test_container_run_create_raises_docker_error(
-            self, mock_create, mock_pull, mock_get_volume_status,
+            self, mock_create, mock_pull, mock_is_volume_available,
             mock_attach_volume, mock_detach_volume,
-            mock_list_by_container, mock_list_by_volume,
+            mock_list_by_container, mock_list_by_volume, mock_count,
             mock_save, mock_spawn_n,
             mock_event_finish, mock_event_start):
         container = Container(self.context, **utils.get_test_container())
         image = {'image': 'repo', 'path': 'out_path', 'driver': 'glance',
                  'repo': 'test', 'tag': 'testtag'}
         mock_pull.return_value = image, True
-        mock_get_volume_status.return_value = 'available'
+        mock_is_volume_available.return_value = True, False
         mock_create.side_effect = exception.DockerError(
             message="Docker Error occurred")
         mock_spawn_n.side_effect = lambda f, *x, **y: f(*x, **y)
         self.compute_manager._resource_tracker = FakeResourceTracker()
         networks = []
-        volumes = [FakeVolumeMapping()]
+        volumes = {container.uuid: [FakeVolumeMapping()]}
         self.assertRaises(
             exception.DockerError,
             self.compute_manager.container_create,
@@ -592,14 +622,81 @@ class TestManager(base.TestCase):
         self.assertEqual('Error', container.status)
         self.assertEqual('Docker Error occurred', container.status_reason)
         mock_pull.assert_any_call(self.context, container.image, '',
-                                  'always', 'glance')
+                                  'always', 'glance', registry=None)
         mock_create.assert_called_once_with(
             self.context, container, image, networks, volumes)
         mock_attach_volume.assert_called_once()
         mock_detach_volume.assert_called_once()
-        mock_get_volume_status.assert_called_once()
+        mock_is_volume_available.assert_called_once()
         self.assertEqual(0, len(FakeVolumeMapping.volumes))
 
+    @mock.patch.object(ContainerActionEvent, 'event_start')
+    @mock.patch.object(ContainerActionEvent, 'event_finish')
+    @mock.patch('zun.common.utils.spawn_n')
+    @mock.patch.object(objects.Capsule, 'save')
+    @mock.patch.object(objects.CapsuleContainer, 'list_by_capsule_id')
+    @mock.patch.object(objects.CapsuleInitContainer, 'list_by_capsule_id')
+    @mock.patch.object(VolumeMapping, 'count',
+                       side_effect=FakeVolumeMapping.count)
+    @mock.patch.object(VolumeMapping, 'list_by_container',
+                       side_effect=FakeVolumeMapping.list_by_container)
+    @mock.patch.object(fake_driver, 'pull_image')
+    @mock.patch.object(fake_driver, 'detach_volume')
+    @mock.patch.object(fake_driver, 'attach_volume')
+    @mock.patch.object(fake_driver, 'is_volume_available')
+    @mock.patch.object(fake_driver, 'create_capsule')
+    @mock.patch.object(fake_driver, 'start')
+    def test_capsule_create(
+            self, mock_start, mock_create,
+            mock_is_volume_available, mock_attach_volume,
+            mock_detach_volume, mock_pull, mock_list_by_container, mock_count,
+            mock_init_container_list, mock_capsule_container_list,
+            mock_save, mock_spawn_n, mock_event_finish, mock_event_start):
+        capsule = objects.Capsule(self.context, **utils.get_test_container())
+        image = {'image': 'repo', 'path': 'out_path', 'driver': 'glance'}
+        mock_create.return_value = capsule
+        container_uuid = uuidutils.generate_uuid()
+        container = objects.CapsuleContainer(
+            self.context, **utils.get_test_container(uuid=container_uuid))
+        mock_capsule_container_list.return_value = [container]
+        container_uuid = uuidutils.generate_uuid()
+        init_container = objects.CapsuleInitContainer(
+            self.context, **utils.get_test_container(uuid=container_uuid))
+        mock_init_container_list.return_value = [init_container]
+        mock_pull.return_value = image, False
+        mock_is_volume_available.return_value = True, False
+        mock_spawn_n.side_effect = lambda f, *x, **y: f(*x, **y)
+        capsule.status = 'Running'
+        self.compute_manager._resource_tracker = FakeResourceTracker()
+        networks = []
+        volmap1 = FakeVolumeMapping()
+        volmap1.uuid = 'fake-uuid-1'
+        volmap2 = FakeVolumeMapping()
+        volmap2.uuid = 'fake-uuid-2'
+        volumes = {container.uuid: [volmap1],
+                   init_container.uuid: [volmap2]}
+
+        def attach_volume(context, volmap):
+            volmap.connection_info = 'fake-info'
+
+        mock_attach_volume.side_effect = attach_volume
+        self.compute_manager.container_create(
+            self.context,
+            requested_networks=networks,
+            requested_volumes=volumes,
+            container=capsule,
+            limits=None, run=True)
+        mock_save.assert_called_with(self.context)
+        mock_pull.assert_any_call(self.context, capsule.image, '',
+                                  'always', 'glance', registry=None)
+        mock_create.assert_called_once_with(self.context, capsule, image,
+                                            networks, volumes)
+        mock_start.assert_called_once_with(self.context, capsule)
+        mock_attach_volume.assert_called_once()
+        mock_detach_volume.assert_not_called()
+        self.assertEqual(2, mock_is_volume_available.call_count)
+        self.assertEqual(2, len(FakeVolumeMapping.volumes))
+
     @mock.patch.object(FakeResourceTracker,
                        'remove_usage_from_container')
     @mock.patch.object(Container, 'destroy')
@@ -665,57 +762,6 @@ class TestManager(base.TestCase):
         mock_remove_usage.assert_called_once_with(self.context, container,
                                                   True)
 
-    @mock.patch.object(FakeResourceTracker,
-                       'remove_usage_from_container')
-    @mock.patch.object(Container, 'destroy')
-    @mock.patch.object(manager.Manager, '_fail_container')
-    @mock.patch.object(manager.Manager, '_delete_sandbox')
-    @mock.patch.object(Container, 'save')
-    @mock.patch.object(fake_driver, 'delete')
-    def test_container_delete_sandbox_failed(self, mock_delete, mock_save,
-                                             mock_delete_sandbox,
-                                             mock_fail, mock_destroy,
-                                             mock_remove_usage):
-        self.compute_manager.use_sandbox = True
-        container = Container(self.context, **utils.get_test_container())
-        container.set_sandbox_id("sandbox_id")
-        mock_delete_sandbox.side_effect = exception.ZunException(
-            message="Unexpected exception")
-        self.assertRaises(exception.ZunException,
-                          self.compute_manager._do_container_delete,
-                          self.context, container, False)
-        mock_save.assert_called_with(self.context)
-        mock_fail.assert_called_with(self.context,
-                                     container, 'Unexpected exception')
-        mock_destroy.assert_not_called()
-        mock_remove_usage.assert_not_called()
-
-    @mock.patch.object(FakeResourceTracker,
-                       'remove_usage_from_container')
-    @mock.patch.object(Container, 'destroy')
-    @mock.patch.object(manager.Manager, '_fail_container')
-    @mock.patch.object(manager.Manager, '_delete_sandbox')
-    @mock.patch.object(Container, 'save')
-    @mock.patch.object(VolumeMapping, 'list_by_container')
-    @mock.patch.object(fake_driver, 'delete')
-    def test_container_delete_sandbox_failed_force(self, mock_delete,
-                                                   mock_list_by_container,
-                                                   mock_save,
-                                                   mock_delete_sandbox,
-                                                   mock_fail, mock_destroy,
-                                                   mock_remove_usage):
-        mock_list_by_container.return_value = []
-        self.compute_manager.use_sandbox = True
-        container = Container(self.context, **utils.get_test_container())
-        container.set_sandbox_id("sandbox_id")
-        mock_delete_sandbox.side_effect = exception.ZunException(
-            message="Unexpected exception")
-        self.compute_manager._do_container_delete(self.context, container,
-                                                  True)
-        mock_save.assert_called_with(self.context)
-        mock_fail.assert_called_with(self.context,
-                                     container, 'Unexpected exception')
-
     @mock.patch.object(fake_driver, 'show')
     def test_container_show(self, mock_show):
         container = Container(self.context, **utils.get_test_container())
@@ -750,7 +796,8 @@ class TestManager(base.TestCase):
         mock_get_network_info.return_value = []
         mock_get_vol_info.return_value = []
         mock_check.return_value = True
-        self.compute_manager._do_container_rebuild(self.context, container)
+        self.compute_manager._do_container_rebuild(
+            self.context, container, False)
         mock_save.assert_called_with(self.context)
         self.assertTrue(mock_create.called)
         mock_delete.assert_called_once_with(self.context, container, True)
@@ -776,7 +823,7 @@ class TestManager(base.TestCase):
         container = Container(self.context, **utils.get_test_container())
         self.assertRaises(exception.PortNotFound,
                           self.compute_manager._do_container_rebuild,
-                          self.context, container)
+                          self.context, container, True)
         mock_fail.assert_called_with(self.context,
                                      container, str(fake_exc))
         mock_event_start.assert_called_once()
@@ -1095,6 +1142,14 @@ class TestManager(base.TestCase):
                           self.compute_manager.container_update,
                           self.context, container, {})
 
+    @mock.patch.object(fake_driver, 'update')
+    def test_container_update_failed_resources(self, mock_update):
+        container = Container(self.context, **utils.get_test_container())
+        mock_update.side_effect = exception.ResourcesUnavailable
+        self.assertRaises(exception.ResourcesUnavailable,
+                          self.compute_manager.container_update,
+                          self.context, container, {})
+
     @mock.patch.object(fake_driver, 'get_websocket_url')
     @mock.patch.object(Container, 'save')
     def test_container_attach_successful(self, mock_save,
@@ -1134,11 +1189,13 @@ class TestManager(base.TestCase):
     @mock.patch.object(fake_driver, 'pull_image')
     def test_image_pull(self, mock_pull, mock_save, mock_inspect):
         image = Image(self.context, **utils.get_test_image())
-        ret = {'image': 'repo', 'path': 'out_path', 'driver': 'glance'}
+        ret = {'image': 'repo', 'path': 'out_path', 'driver': 'docker'}
         mock_pull.return_value = ret, True
-        mock_inspect.return_value = {'Id': 'fake-id', 'Size': 512}
+        mock_inspect.return_value = {'Id': 'fake-id', 'Size': 512,
+                                     'RepoTags': ['image1:latest']}
         self.compute_manager._do_image_pull(self.context, image)
-        mock_pull.assert_any_call(self.context, image.repo, image.tag)
+        mock_pull.assert_any_call(self.context, image.repo, image.tag,
+                                  driver_name='docker')
         mock_save.assert_called_once()
         mock_inspect.assert_called_once_with(image.repo + ":" + image.tag)
 
@@ -1150,11 +1207,14 @@ class TestManager(base.TestCase):
                                    mock_inspect, mock_load):
         image = Image(self.context, **utils.get_test_image())
         repo_tag = image.repo + ":" + image.tag
-        ret = {'image': 'repo', 'path': 'out_path', 'driver': 'glance'}
+        ret = {'image': 'repo', 'path': 'out_path', 'driver': 'docker',
+               'tags': ['latest']}
         mock_pull.return_value = ret, False
-        mock_inspect.return_value = {'Id': 'fake-id', 'Size': 512}
+        mock_inspect.return_value = {'Id': 'fake-id', 'Size': 512,
+                                     'RepoTags': ['image1:latest']}
         self.compute_manager._do_image_pull(self.context, image)
-        mock_pull.assert_any_call(self.context, image.repo, image.tag)
+        mock_pull.assert_any_call(self.context, image.repo, image.tag,
+                                  driver_name='docker')
         mock_save.assert_called_once()
         mock_inspect.assert_called_once_with(repo_tag)
         mock_load.assert_called_once_with(ret['path'])
@@ -1164,11 +1224,13 @@ class TestManager(base.TestCase):
     @mock.patch.object(fake_driver, 'pull_image')
     def test_image_pull_tag_is_none(self, mock_pull, mock_save, mock_inspect):
         image = Image(self.context, **utils.get_test_image(tag=None))
-        ret = {'image': 'repo', 'path': 'out_path', 'driver': 'glance'}
+        ret = {'image': 'repo', 'path': 'out_path', 'driver': 'docker'}
         mock_pull.return_value = ret, True
-        mock_inspect.return_value = {'Id': 'fake-id', 'Size': 512}
+        mock_inspect.return_value = {'Id': 'fake-id', 'Size': 512,
+                                     'RepoTags': ['image1:latest']}
         self.compute_manager._do_image_pull(self.context, image)
-        mock_pull.assert_any_call(self.context, image.repo, image.tag)
+        mock_pull.assert_any_call(self.context, image.repo, None,
+                                  driver_name='docker')
         mock_save.assert_called_once()
         mock_inspect.assert_called_once_with(image.repo)
 
@@ -1318,43 +1380,49 @@ class TestManager(base.TestCase):
         self.assertIsNone(mock_event_finish.call_args[1]['exc_val'])
         self.assertIsNone(mock_event_finish.call_args[1]['exc_tb'])
 
-    @mock.patch.object(fake_driver, 'get_volume_status')
+    @mock.patch.object(fake_driver, 'is_volume_available')
     @mock.patch.object(manager.Manager, '_fail_container')
     def test_wait_for_volumes_available(self, mock_fail,
-                                        mock_get_volume_status):
-        mock_get_volume_status.return_value = 'available'
+                                        mock_is_volume_available):
+        mock_is_volume_available.return_value = True, False
         container = Container(self.context, **utils.get_test_container())
-        volumes = [FakeVolumeMapping()]
+        FakeVolumeMapping.container_uuid = container.uuid
+        volumes = {container.uuid: [FakeVolumeMapping()]}
         self.compute_manager._wait_for_volumes_available(self.context,
                                                          volumes,
                                                          container)
-        mock_get_volume_status.assert_called_once()
+        mock_is_volume_available.assert_called_once()
         mock_fail.assert_not_called()
 
     @mock.patch.object(fake_driver, 'delete_volume')
-    @mock.patch.object(fake_driver, 'get_volume_status')
+    @mock.patch.object(fake_driver, 'is_volume_available')
     @mock.patch.object(manager.Manager, '_fail_container')
     def test_wait_for_volumes_available_failed(self, mock_fail,
-                                               mock_get_volume_status,
+                                               mock_is_volume_available,
                                                mock_delete_volume):
-        mock_get_volume_status.return_value = 'error'
+        mock_is_volume_available.return_value = False, True
         container = Container(self.context, **utils.get_test_container())
+        FakeVolumeMapping.container_uuid = container.uuid
         volume = FakeVolumeMapping()
         volume.auto_remove = True
-        volumes = [volume]
+        volumes = {container.uuid: [volume]}
         self.assertRaises(exception.Conflict,
                           self.compute_manager._wait_for_volumes_available,
                           self.context, volumes, container, timeout=2)
-        self.assertTrue(mock_get_volume_status.called)
+        self.assertTrue(mock_is_volume_available.called)
         self.assertTrue(mock_fail.called)
         self.assertTrue(mock_delete_volume.called)
 
-    @mock.patch.object(Network, 'save')
     @mock.patch.object(fake_driver, 'create_network')
-    def test_network_create(self, mock_create, mock_save):
+    def test_network_create(self, mock_create):
         network = Network(self.context, **utils.get_test_network())
         ret = ({'Id': '0eeftestnetwork'})
         mock_create.return_value = ret
         self.compute_manager.network_create(self.context, network)
         mock_create.assert_any_call(self.context, network)
-        mock_save.assert_called_once()
+
+    @mock.patch.object(fake_driver, 'delete_network')
+    def test_network_delete(self, mock_delete):
+        network = Network(self.context, **utils.get_test_network())
+        self.compute_manager.network_delete(self.context, network)
+        mock_delete.assert_any_call(self.context, network)
