@@ -10,6 +10,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+from collections import defaultdict
 from docker import errors
 import mock
 
@@ -39,6 +40,8 @@ CONF = conf.CONF
 _numa_node = {
     'id': 0,
     'cpuset': [8],
+    'mem_available': 32768,
+    'mem_total': 32768,
     'pinned_cpus': []
 }
 
@@ -94,6 +97,8 @@ class TestDockerDriver(base.DriverTestCase):
         self.driver.images(repo='test')
         self.mock_docker.images.assert_called_once_with('test', False)
 
+    @mock.patch('neutronclient.v2_0.client.Client.create_security_group')
+    @mock.patch('zun.network.neutron.NeutronAPI.expose_ports')
     @mock.patch('zun.network.kuryr_network.KuryrNetwork'
                 '.connect_container_to_network')
     @mock.patch('zun.network.kuryr_network.KuryrNetwork'
@@ -104,7 +109,9 @@ class TestDockerDriver(base.DriverTestCase):
             self, mock_save,
             mock_get_security_group_ids,
             mock_create_or_update_port,
-            mock_connect):
+            mock_connect,
+            mock_expose_ports,
+            mock_create_security_group):
         self.mock_docker.create_host_config = mock.Mock(
             return_value={'Id1': 'val1', 'key2': 'val2'})
         self.mock_docker.create_container = mock.Mock(
@@ -117,10 +124,13 @@ class TestDockerDriver(base.DriverTestCase):
         image = {'path': '', 'image': '', 'repo': 'test', 'tag': 'test'}
         mock_container = self.mock_default_container
         mock_container.status = 'Creating'
+        mock_container.healthcheck = {}
         networks = [{'network': 'fake-network'}]
-        volumes = []
+        volumes = {}
         fake_port = {'mac_address': 'fake_mac'}
         mock_create_or_update_port.return_value = ([], fake_port)
+        mock_create_security_group.return_value = {
+            'security_group': {'id': 'fake-id'}}
         # DockerDriver with supported storage driver - overlay2
         self.driver._host.sp_disk_quota = True
         self.driver._host.storage_driver = 'overlay2'
@@ -135,6 +145,7 @@ class TestDockerDriver(base.DriverTestCase):
         host_config['binds'] = {}
         host_config['network_mode'] = 'fake-network'
         host_config['storage_opt'] = {'size': '20G'}
+        host_config['privileged'] = False
         self.mock_docker.create_host_config.assert_called_once_with(
             **host_config)
 
@@ -159,6 +170,8 @@ class TestDockerDriver(base.DriverTestCase):
         self.assertEqual(result_container.status,
                          consts.CREATED)
 
+    @mock.patch('neutronclient.v2_0.client.Client.create_security_group')
+    @mock.patch('zun.network.neutron.NeutronAPI.expose_ports')
     @mock.patch('zun.network.kuryr_network.KuryrNetwork'
                 '.connect_container_to_network')
     @mock.patch('zun.network.kuryr_network.KuryrNetwork'
@@ -169,7 +182,9 @@ class TestDockerDriver(base.DriverTestCase):
             self, mock_save,
             mock_get_security_group_ids,
             mock_create_or_update_port,
-            mock_connect):
+            mock_connect,
+            mock_expose_ports,
+            mock_create_security_group):
         self.mock_docker.create_host_config = mock.Mock(
             return_value={'Id1': 'val1', 'key2': 'val2'})
         self.mock_docker.create_container = mock.Mock(
@@ -182,10 +197,13 @@ class TestDockerDriver(base.DriverTestCase):
         image = {'path': '', 'image': '', 'repo': 'test', 'tag': 'test'}
         mock_container = self.mock_default_container
         mock_container.status = 'Creating'
+        mock_container.healthcheck = {}
         networks = [{'network': 'fake-network'}]
-        volumes = []
+        volumes = {}
         fake_port = {'mac_address': 'fake_mac'}
         mock_create_or_update_port.return_value = ([], fake_port)
+        mock_create_security_group.return_value = {
+            'security_group': {'id': 'fake-id'}}
         # DockerDriver with supported storage driver - overlay2
         self.driver._host.sp_disk_quota = True
         self.driver._host.storage_driver = 'devicemapper'
@@ -201,6 +219,7 @@ class TestDockerDriver(base.DriverTestCase):
         host_config['binds'] = {}
         host_config['network_mode'] = 'fake-network'
         host_config['storage_opt'] = {'size': '20G'}
+        host_config['privileged'] = False
         self.mock_docker.create_host_config.assert_called_once_with(
             **host_config)
 
@@ -225,6 +244,8 @@ class TestDockerDriver(base.DriverTestCase):
         self.assertEqual(result_container.status,
                          consts.CREATED)
 
+    @mock.patch('neutronclient.v2_0.client.Client.create_security_group')
+    @mock.patch('zun.network.neutron.NeutronAPI.expose_ports')
     @mock.patch('zun.network.kuryr_network.KuryrNetwork'
                 '.connect_container_to_network')
     @mock.patch('zun.network.kuryr_network.KuryrNetwork'
@@ -235,7 +256,9 @@ class TestDockerDriver(base.DriverTestCase):
             self, mock_save,
             mock_get_security_group_ids,
             mock_create_or_update_port,
-            mock_connect):
+            mock_connect,
+            mock_expose_ports,
+            mock_create_security_group):
         CONF.set_override("docker_remote_api_version", "1.24", "docker")
         self.mock_docker.create_host_config = mock.Mock(
             return_value={'Id1': 'val1', 'key2': 'val2'})
@@ -249,11 +272,14 @@ class TestDockerDriver(base.DriverTestCase):
         image = {'path': '', 'image': '', 'repo': 'test', 'tag': 'test'}
         mock_container = self.mock_default_container
         mock_container.status = 'Creating'
+        mock_container.healthcheck = {}
         mock_container.runtime = None
         networks = [{'network': 'fake-network'}]
-        volumes = []
+        volumes = {}
         fake_port = {'mac_address': 'fake_mac'}
         mock_create_or_update_port.return_value = ([], fake_port)
+        mock_create_security_group.return_value = {
+            'security_group': {'id': 'fake-id'}}
         result_container = self.driver.create(self.context, mock_container,
                                               image, networks, volumes)
         host_config = {}
@@ -265,6 +291,7 @@ class TestDockerDriver(base.DriverTestCase):
         host_config['binds'] = {}
         host_config['network_mode'] = 'fake-network'
         host_config['storage_opt'] = {'size': '20G'}
+        host_config['privileged'] = False
         self.mock_docker.create_host_config.assert_called_once_with(
             **host_config)
 
@@ -315,7 +342,7 @@ class TestDockerDriver(base.DriverTestCase):
         mock_container.status = 'Creating'
         mock_container.runtime = 'runc'
         networks = [{'network': 'fake-network'}]
-        volumes = []
+        volumes = {}
         fake_port = {'mac_address': 'fake_mac'}
         mock_create_or_update_port.return_value = ([], fake_port)
         with self.assertRaisesRegex(
@@ -329,26 +356,42 @@ class TestDockerDriver(base.DriverTestCase):
 
     @mock.patch('zun.container.docker.driver.DockerDriver'
                 '._cleanup_network_for_container')
-    def test_delete_success(self, mock_cleanup_network_for_container):
+    @mock.patch('zun.container.docker.driver.DockerDriver'
+                '._cleanup_exposed_ports')
+    def test_delete_success(self, mock_cleanup_network_for_container,
+                            mock_cleanup_exposed_ports):
         self.mock_docker.remove_container = mock.Mock()
         mock_container = self.mock_default_container
         self.driver.delete(self.context, mock_container, True)
         self.assertTrue(mock_cleanup_network_for_container.called)
+        self.assertTrue(mock_cleanup_exposed_ports.called)
         self.mock_docker.remove_container.assert_called_once_with(
             mock_container.container_id, force=True)
 
-    def test_delete_fail_no_result(self):
+    @mock.patch('zun.container.docker.driver.DockerDriver'
+                '._cleanup_network_for_container')
+    @mock.patch('zun.container.docker.driver.DockerDriver'
+                '._cleanup_exposed_ports')
+    def test_delete_fail_no_result(self, mock_cleanup_network_for_container,
+                                   mock_cleanup_exposed_ports):
         with mock.patch.object(errors.APIError, '__str__',
                                return_value='404 Not Found') as mock_init:
             self.mock_docker.remove_container = mock.Mock(
                 side_effect=errors.APIError('Error', '', ''))
             mock_container = mock.MagicMock()
             self.driver.delete(self.context, mock_container, True)
+            self.assertTrue(mock_cleanup_network_for_container.called)
+            self.assertTrue(mock_cleanup_exposed_ports.called)
             self.mock_docker.remove_container.assert_called_once_with(
                 mock_container.container_id, force=True)
             self.assertEqual(1, mock_init.call_count)
 
-    def test_delete_fail_raise_error(self):
+    @mock.patch('zun.container.docker.driver.DockerDriver'
+                '._cleanup_network_for_container')
+    @mock.patch('zun.container.docker.driver.DockerDriver'
+                '._cleanup_exposed_ports')
+    def test_delete_fail_raise_error(self, mock_cleanup_network_for_container,
+                                     mock_cleanup_exposed_ports):
         with mock.patch.object(errors.APIError, '__str__',
                                return_value='test') as mock_init:
             self.mock_docker.remove_container = mock.Mock(
@@ -357,6 +400,8 @@ class TestDockerDriver(base.DriverTestCase):
             self.assertRaises(errors.APIError, self.driver.delete,
                               self.context, mock_container,
                               True)
+            self.assertTrue(mock_cleanup_network_for_container.called)
+            self.assertTrue(mock_cleanup_exposed_ports.called)
             self.mock_docker.remove_container.assert_called_once_with(
                 mock_container.container_id, force=True)
             self.assertEqual(2, mock_init.call_count)
@@ -371,8 +416,6 @@ class TestDockerDriver(base.DriverTestCase):
         uuid2 = uuidutils.generate_uuid()
         mock_container_list = [
             {'Names': ['/%s%s' % (consts.NAME_PREFIX, uuid)]},
-            {'Names': ['/%s%s' % (consts.SANDBOX_NAME_PREFIX,
-                                  uuidutils.generate_uuid())]},
             {'Names': ['/%s%s' % (consts.NAME_PREFIX, uuid2)]}]
         uuids = self.driver._get_container_uuids(mock_container_list)
         self.assertEqual(sorted([uuid, uuid2]), sorted(uuids))
@@ -452,23 +495,23 @@ class TestDockerDriver(base.DriverTestCase):
             self.context, status='Stopped')
         conf.CONF.set_override('host', 'host2')
         with mock.patch.object(self.driver, 'list') as mock_list:
-            mock_list.return_value = [mock_container_2]
+            mock_list.return_value = ([mock_container_2], [])
             self.assertEqual(mock_container.host, 'host1')
             self.assertEqual(mock_container.status, 'Running')
             self.driver.update_containers_states(
-                self.context, [mock_container])
+                self.context, [mock_container], mock.Mock())
             self.assertEqual(mock_container.host, 'host2')
             self.assertEqual(mock_container.status, 'Stopped')
 
-    @mock.patch('zun.compute.api.API.container_rebuild')
-    def test_heal_with_rebuilding_container(self, mock_container_rebuild):
+    def test_heal_with_rebuilding_container(self):
+        mock_compute_manager = mock.Mock()
         mock_container = obj_utils.get_test_container(
             self.context, status='Running',
             auto_heal=True, task_state=None)
         self.driver.heal_with_rebuilding_container(
+            self.context, mock_container, mock_compute_manager)
+        mock_compute_manager.container_rebuild.assert_called_once_with(
             self.context, mock_container)
-        mock_container_rebuild.assert_called_once_with(self.context,
-                                                       mock_container)
 
     @mock.patch('zun.compute.api.API.container_rebuild')
     def test_heal_with_rebuilding_exception(self, mock_container_rebuild):
@@ -476,7 +519,7 @@ class TestDockerDriver(base.DriverTestCase):
         container.status = consts.RUNNING
         mock_container_rebuild.side_effect = Exception
         self.driver.heal_with_rebuilding_container(
-            self.context, container)
+            self.context, container, mock.Mock())
 
     def test_show_success(self):
         self.mock_docker.inspect_container = mock.Mock(
@@ -708,95 +751,6 @@ class TestDockerDriver(base.DriverTestCase):
         self.mock_docker.commit.assert_called_once_with(
             mock_container.container_id, "repo", "tag")
 
-    @mock.patch('zun.network.kuryr_network.KuryrNetwork'
-                '.connect_container_to_network')
-    @mock.patch('zun.network.kuryr_network.KuryrNetwork'
-                '.create_or_update_port')
-    @mock.patch('zun.container.docker.driver.DockerDriver.get_sandbox_name')
-    @mock.patch('zun.common.utils.get_security_group_ids')
-    def test_create_sandbox(self, mock_get_security_group_ids,
-                            mock_get_sandbox_name, mock_create_or_update_port,
-                            mock_connect):
-        sandbox_name = 'my_test_sandbox'
-        mock_get_sandbox_name.return_value = sandbox_name
-        self.mock_docker.create_container = mock.Mock(
-            return_value={'Id': 'val1', 'key1': 'val2'})
-        self.mock_docker.create_networking_config = mock.Mock(
-            return_value={'Id': 'val1', 'key1': 'val2'})
-        fake_host_config = mock.Mock()
-        self.mock_docker.create_host_config.return_value = fake_host_config
-        mock_container = mock.MagicMock()
-        hostname = 'my_hostname'
-        mock_container.hostname = hostname
-        requested_networks = [{'network': 'fake-network'}]
-        requested_volumes = []
-        fake_port = {'mac_address': 'fake_mac'}
-        mock_create_or_update_port.return_value = ([], fake_port)
-        result_sandbox_id = self.driver.create_sandbox(
-            self.context, mock_container, requested_networks,
-            requested_volumes, 'kubernetes/pause')
-        self.mock_docker.create_container.assert_called_once_with(
-            'kubernetes/pause', name=sandbox_name, hostname=sandbox_name,
-            host_config=fake_host_config, volumes=[], mac_address='fake_mac',
-            networking_config={'Id': 'val1', 'key1': 'val2'})
-        self.assertEqual(result_sandbox_id, 'val1')
-
-    @mock.patch('zun.network.kuryr_network.KuryrNetwork'
-                '.connect_container_to_network')
-    @mock.patch('zun.network.kuryr_network.KuryrNetwork'
-                '.create_or_update_port')
-    @mock.patch('zun.container.docker.driver.DockerDriver.get_sandbox_name')
-    @mock.patch('zun.common.utils.get_security_group_ids')
-    def test_create_sandbox_with_long_name(self, mock_get_security_group_ids,
-                                           mock_get_sandbox_name,
-                                           mock_create_or_update_port,
-                                           mock_connect):
-        sandbox_name = 'x' * 100
-        mock_get_sandbox_name.return_value = sandbox_name
-        self.mock_docker.create_container = mock.Mock(
-            return_value={'Id': 'val1', 'key1': 'val2'})
-        self.mock_docker.create_networking_config = mock.Mock(
-            return_value={'Id': 'val1', 'key1': 'val2'})
-        fake_host_config = mock.Mock()
-        self.mock_docker.create_host_config.return_value = fake_host_config
-        mock_container = mock.MagicMock()
-        mock_container.hostname = None
-        requested_networks = [{'network': 'fake-network'}]
-        requested_volumes = []
-        fake_port = {'mac_address': 'fake_mac'}
-        mock_create_or_update_port.return_value = ([], fake_port)
-        result_sandbox_id = self.driver.create_sandbox(
-            self.context, mock_container, requested_networks,
-            requested_volumes, 'kubernetes/pause')
-        self.mock_docker.create_container.assert_called_once_with(
-            'kubernetes/pause', name=sandbox_name, hostname=sandbox_name[:63],
-            host_config=fake_host_config, volumes=[], mac_address='fake_mac',
-            networking_config={'Id': 'val1', 'key1': 'val2'})
-        self.assertEqual(result_sandbox_id, 'val1')
-
-    def test_delete_sandbox(self):
-        self.mock_docker.remove_container = mock.Mock()
-        mock_container = mock.MagicMock()
-        mock_container.get_sandbox_id.return_value = 'test_sandbox_id'
-        self.driver.delete_sandbox(self.context, mock_container)
-        self.mock_docker.remove_container.assert_called_once_with(
-            'test_sandbox_id', force=True)
-
-    def test_stop_sandbox(self):
-        self.mock_docker.stop = mock.Mock()
-        self.driver.stop_sandbox(context=self.context,
-                                 sandbox_id='test_sandbox_id')
-        self.mock_docker.stop.assert_called_once_with('test_sandbox_id')
-
-    def test_get_sandbox_name(self):
-        mock_container = mock.MagicMock(
-            uuid='ea8e2a25-2901-438d-8157-de7ffd68d051')
-        result_sanbox_name = self.driver.get_sandbox_name(mock_container)
-        self.assertEqual(
-            result_sanbox_name,
-            '%sea8e2a25-2901-438d-8157-de7ffd68d051' %
-            consts.SANDBOX_NAME_PREFIX)
-
     def test_get_container_name(self):
         mock_container = mock.MagicMock(
             uuid='ea8e2a25-2901-438d-8157-de7ffd68d051')
@@ -824,19 +778,22 @@ class TestDockerDriver(base.DriverTestCase):
                                               'OSType': 'linux',
                                               'OperatingSystem': 'CentOS',
                                               'KernelVersion': '3.10.0-123',
-                                              'Labels': ['dev.type=product']}
-        (total, running, paused, stopped, cpus, architecture,
-         os_type, os, kernel_version, labels) = self.driver.get_host_info()
-        self.assertEqual(10, total)
-        self.assertEqual(8, running)
-        self.assertEqual(0, paused)
-        self.assertEqual(2, stopped)
-        self.assertEqual(48, cpus)
-        self.assertEqual('x86_64', architecture)
-        self.assertEqual('linux', os_type)
-        self.assertEqual('CentOS', os)
-        self.assertEqual('3.10.0-123', kernel_version)
-        self.assertEqual({"dev.type": "product"}, labels)
+                                              'Labels': ['dev.type=product'],
+                                              'Runtimes': {'runc': {'path':
+                                                           'docker-runc'}},
+                                              'DockerRootDir': 'fake-dir'}
+        host_info = self.driver.get_host_info()
+        self.assertEqual(10, host_info['total_containers'])
+        self.assertEqual(8, host_info['running_containers'])
+        self.assertEqual(0, host_info['paused_containers'])
+        self.assertEqual(2, host_info['stopped_containers'])
+        self.assertEqual(48, host_info['cpus'])
+        self.assertEqual('x86_64', host_info['architecture'])
+        self.assertEqual('linux', host_info['os_type'])
+        self.assertEqual('CentOS', host_info['os'])
+        self.assertEqual('3.10.0-123', host_info['kernel_version'])
+        self.assertEqual({"dev.type": "product"}, host_info['labels'])
+        self.assertEqual('fake-dir', host_info['docker_root_dir'])
 
     def test_get_cpu_used(self):
         self.mock_docker.containers = mock.Mock()
@@ -950,6 +907,10 @@ class TestDockerDriver(base.DriverTestCase):
                                              security_groups=test_sec_group_id)
 
     @mock.patch('zun.common.utils.execute')
+    @mock.patch('zun.container.os_capability.linux.os_capability_linux'
+                '.LinuxHost.get_mem_numa_info')
+    @mock.patch('zun.container.os_capability.linux.os_capability_linux'
+                '.LinuxHost.get_cpu_numa_info')
     @mock.patch('zun.container.docker.driver.DockerDriver'
                 '.get_total_disk_for_container')
     @mock.patch('zun.container.driver.ContainerDriver.get_host_mem')
@@ -958,15 +919,30 @@ class TestDockerDriver(base.DriverTestCase):
     @mock.patch(
         'zun.container.docker.driver.DockerDriver.get_cpu_used')
     def test_get_available_resources(self, mock_cpu_used, mock_info, mock_mem,
-                                     mock_disk, mock_output):
+                                     mock_disk, mock_numa_cpu, mock_numa_mem,
+                                     mock_output):
         self.driver = DockerDriver()
+        numa_cpu_info = defaultdict(list)
+        numa_cpu_info['0'] = [0, 8]
+        mock_numa_cpu.return_value = numa_cpu_info
+        mock_numa_mem.return_value = [1024 * 32]
         mock_output.return_value = LSCPU_ON
         conf.CONF.set_override('floating_cpu_set', "0")
         mock_mem.return_value = (100 * units.Ki, 50 * units.Ki, 50 * units.Ki,
                                  50 * units.Ki)
-        mock_info.return_value = (10, 8, 0, 2, 48, 'x86_64', 'linux',
-                                  'CentOS', '3.10.0-123',
-                                  {'dev.type': 'product'})
+        mock_info.return_value = {'total_containers': 10,
+                                  'running_containers': 8,
+                                  'paused_containers': 0,
+                                  'stopped_containers': 2,
+                                  'cpus': 48,
+                                  'architecture': 'x86_64',
+                                  'os_type': 'linux',
+                                  'os': 'CentOS',
+                                  'kernel_version': '3.10.0-123',
+                                  'labels': {'dev.type': 'product'},
+                                  'runtimes': ['runc'],
+                                  'enable_cpu_pinning': False,
+                                  'docker_root_dir': '/var/lib/docker'}
         mock_cpu_used.return_value = 1.0
         mock_disk.return_value = 80
         node_obj = objects.ComputeNode()
@@ -987,3 +963,4 @@ class TestDockerDriver(base.DriverTestCase):
         self.assertEqual('3.10.0-123', node_obj.kernel_version)
         self.assertEqual({'dev.type': 'product'}, node_obj.labels)
         self.assertEqual(80, node_obj.disk_total)
+        self.assertEqual(['runc'], node_obj.runtimes)
