@@ -18,10 +18,6 @@ from zun.container import driver
 
 class FakeDriver(driver.ContainerDriver):
     """Fake driver for testing."""
-    capabilities = {
-        "support_sandbox": True,
-        "support_standalone": True,
-    }
 
     def __init__(self):
         super(FakeDriver, self).__init__()
@@ -63,6 +59,12 @@ class FakeDriver(driver.ContainerDriver):
     def show(self, context, container):
         pass
 
+    def create_capsule(self, context, capsule, **kwargs):
+        pass
+
+    def delete_capsule(self, context, capsule, **kwargs):
+        pass
+
     @check_container_id
     def reboot(self, context, container):
         pass
@@ -103,12 +105,6 @@ class FakeDriver(driver.ContainerDriver):
     def resize(self, context, container, height, weight):
         pass
 
-    def create_sandbox(self, context, name, **kwargs):
-        pass
-
-    def delete_sandbox(self, context, id):
-        pass
-
     def get_addresses(self, context, container):
         pass
 
