@@ -18,6 +18,7 @@ import mock
 from oslo_utils import uuidutils
 from testtools.matchers import HasLength
 
+from zun.common import consts
 from zun import objects
 from zun.tests.unit.db import base
 from zun.tests.unit.db import utils
@@ -27,26 +28,57 @@ class TestContainerObject(base.DbTestCase):
 
     def setUp(self):
         super(TestContainerObject, self).setUp()
-        self.fake_container = utils.get_test_container()
+        self.fake_cpuset = utils.get_cpuset_dict()
+        self.fake_container = utils.get_test_container(
+            cpuset=self.fake_cpuset, cpu_policy='dedicated')
+        self.fake_container.pop('capsule_id')
 
     def test_get_by_uuid(self):
         uuid = self.fake_container['uuid']
+        container_type = self.fake_container['container_type']
         with mock.patch.object(self.dbapi, 'get_container_by_uuid',
                                autospec=True) as mock_get_container:
             mock_get_container.return_value = self.fake_container
             container = objects.Container.get_by_uuid(self.context, uuid)
-            mock_get_container.assert_called_once_with(self.context, uuid)
+            mock_get_container.assert_called_once_with(
+                self.context, container_type, uuid)
             self.assertEqual(self.context, container._context)
 
     def test_get_by_name(self):
         name = self.fake_container['name']
+        container_type = self.fake_container['container_type']
         with mock.patch.object(self.dbapi, 'get_container_by_name',
                                autospec=True) as mock_get_container:
             mock_get_container.return_value = self.fake_container
             container = objects.Container.get_by_name(self.context, name)
-            mock_get_container.assert_called_once_with(self.context, name)
+            mock_get_container.assert_called_once_with(
+                self.context, container_type, name)
             self.assertEqual(self.context, container._context)
 
+    def test_get_container_any_type(self):
+        mapping = {
+            consts.TYPE_CONTAINER: objects.Container,
+            consts.TYPE_CAPSULE: objects.Capsule,
+            consts.TYPE_CAPSULE_CONTAINER: objects.CapsuleContainer,
+            consts.TYPE_CAPSULE_INIT_CONTAINER: objects.CapsuleInitContainer,
+        }
+        for type, container_cls in mapping.items():
+            self._test_get_container_any_type(type, container_cls)
+
+    def _test_get_container_any_type(self, container_type, container_cls):
+        fake_container = utils.get_test_container(
+            container_type=container_type)
+        uuid = fake_container['uuid']
+        with mock.patch.object(self.dbapi, 'get_container_by_uuid',
+                               autospec=True) as mock_get_container:
+            mock_get_container.return_value = fake_container
+            container = objects.Container.get_container_any_type(
+                self.context, uuid)
+            mock_get_container.assert_called_once_with(
+                self.context, consts.TYPE_ANY, uuid)
+            self.assertEqual(self.context, container._context)
+            self.assertIsInstance(container, container_cls)
+
     def test_list(self):
         with mock.patch.object(self.dbapi, 'list_containers',
                                autospec=True) as mock_get_list:
@@ -64,7 +96,8 @@ class TestContainerObject(base.DbTestCase):
             containers = objects.Container.list_by_host(self.context,
                                                         'test_host')
             mock_get_list.assert_called_once_with(
-                self.context, {'host': 'test_host'}, None, None, None, None)
+                self.context, consts.TYPE_CONTAINER, {'host': 'test_host'},
+                None, None, None, None)
             self.assertThat(containers, HasLength(1))
             self.assertIsInstance(containers[0], objects.Container)
             self.assertEqual(self.context, containers[0]._context)
@@ -81,6 +114,7 @@ class TestContainerObject(base.DbTestCase):
             self.assertIsInstance(containers[0], objects.Container)
             self.assertEqual(self.context, containers[0]._context)
             mock_get_list.assert_called_once_with(self.context,
+                                                  consts.TYPE_CONTAINER,
                                                   filters=filt,
                                                   limit=None, marker=None,
                                                   sort_key=None, sort_dir=None)
@@ -89,7 +123,10 @@ class TestContainerObject(base.DbTestCase):
         with mock.patch.object(self.dbapi, 'create_container',
                                autospec=True) as mock_create_container:
             mock_create_container.return_value = self.fake_container
-            container = objects.Container(self.context, **self.fake_container)
+            container_dict = dict(self.fake_container)
+            container_dict['cpuset'] = objects.container.Cpuset._from_dict(
+                container_dict['cpuset'])
+            container = objects.Container(self.context, **container_dict)
             container.create(self.context)
             mock_create_container.assert_called_once_with(self.context,
                                                           self.fake_container)
@@ -99,7 +136,10 @@ class TestContainerObject(base.DbTestCase):
         with mock.patch.object(self.dbapi, 'create_container',
                                autospec=True) as mock_create_container:
             mock_create_container.return_value = self.fake_container
-            container = objects.Container(self.context, **self.fake_container)
+            container_dict = dict(self.fake_container)
+            container_dict['cpuset'] = objects.container.Cpuset._from_dict(
+                container_dict['cpuset'])
+            container = objects.Container(self.context, **container_dict)
             self.assertTrue(hasattr(container, 'status_reason'))
             container.status_reason = "Docker Error happened"
             container.create(self.context)
@@ -109,6 +149,7 @@ class TestContainerObject(base.DbTestCase):
 
     def test_destroy(self):
         uuid = self.fake_container['uuid']
+        container_type = self.fake_container['container_type']
         with mock.patch.object(self.dbapi, 'get_container_by_uuid',
                                autospec=True) as mock_get_container:
             mock_get_container.return_value = self.fake_container
@@ -116,12 +157,15 @@ class TestContainerObject(base.DbTestCase):
                                    autospec=True) as mock_destroy_container:
                 container = objects.Container.get_by_uuid(self.context, uuid)
                 container.destroy()
-                mock_get_container.assert_called_once_with(self.context, uuid)
-                mock_destroy_container.assert_called_once_with(None, uuid)
+                mock_get_container.assert_called_once_with(
+                    self.context, container_type, uuid)
+                mock_destroy_container.assert_called_once_with(
+                    None, container_type, uuid)
                 self.assertEqual(self.context, container._context)
 
     def test_save(self):
         uuid = self.fake_container['uuid']
+        container_type = self.fake_container['container_type']
         with mock.patch.object(self.dbapi, 'get_container_by_uuid',
                                autospec=True) as mock_get_container:
             mock_get_container.return_value = self.fake_container
@@ -133,9 +177,10 @@ class TestContainerObject(base.DbTestCase):
                 container.environment = {"key1": "val", "key2": "val2"}
                 container.save()
 
-                mock_get_container.assert_called_once_with(self.context, uuid)
+                mock_get_container.assert_called_once_with(
+                    self.context, container_type, uuid)
                 mock_update_container.assert_called_once_with(
-                    None, uuid,
+                    None, container_type, uuid,
                     {'image': 'container.img',
                      'environment': {"key1": "val", "key2": "val2"},
                      'memory': '512m'})
@@ -143,11 +188,12 @@ class TestContainerObject(base.DbTestCase):
 
     def test_refresh(self):
         uuid = self.fake_container['uuid']
+        container_type = self.fake_container['container_type']
         new_uuid = uuidutils.generate_uuid()
         returns = [dict(self.fake_container, uuid=uuid),
                    dict(self.fake_container, uuid=new_uuid)]
-        expected = [mock.call(self.context, uuid),
-                    mock.call(self.context, uuid)]
+        expected = [mock.call(self.context, container_type, uuid),
+                    mock.call(self.context, container_type, uuid)]
         with mock.patch.object(self.dbapi, 'get_container_by_uuid',
                                side_effect=returns,
                                autospec=True) as mock_get_container:
