@@ -50,6 +50,25 @@ class FilterSchedulerTestCase(base.TestCase):
         self.driver.servicegroup_api.service_is_up = mock.Mock(
             return_value=True)
         mock_list_by_binary.side_effect = _return_services
+        numa_topology = {
+            "nodes": [
+                {
+                    "id": 0,
+                    "cpuset": [1, 2, 3, 4],
+                    "pinned_cpus": [],
+                    "mem_total": 1024 * 64,
+                    "mem_available": 1024 * 64
+                },
+                {
+                    "id": 1,
+                    "cpuset": [5, 6, 7, 8],
+                    "pinned_cpus": [],
+                    "mem_total": 1024 * 64,
+                    "mem_available": 1024 * 64
+                }
+            ]
+        }
+        numa = objects.numa.NUMATopology._from_dict(numa_topology)
         test_container = utils.get_test_container()
         containers = [objects.Container(self.context, **test_container)]
         node1 = objects.ComputeNode(self.context)
@@ -58,52 +77,64 @@ class FilterSchedulerTestCase(base.TestCase):
         node1.mem_total = 1024 * 128
         node1.mem_used = 1024 * 4
         node1.mem_free = 1024 * 124
+        node1.mem_available = 1024 * 124
         node1.disk_total = 80
         node1.disk_used = 20
         node1.hostname = 'host1'
-        node1.numa_topology = None
+        node1.numa_topology = numa
         node1.labels = {}
         node1.pci_device_pools = None
         node1.disk_quota_supported = True
+        node1.runtimes = ['runc']
+        node1.enable_cpu_pinning = False
         node2 = objects.ComputeNode(self.context)
         node2.cpus = 48
         node2.cpu_used = 0.0
         node2.mem_total = 1024 * 128
         node2.mem_used = 1024 * 4
         node2.mem_free = 1024 * 124
+        node2.mem_available = 1024 * 124
         node2.disk_total = 80
         node2.disk_used = 20
         node2.hostname = 'host2'
-        node2.numa_topology = None
+        node2.numa_topology = numa
         node2.labels = {}
         node2.pci_device_pools = None
         node2.disk_quota_supported = True
+        node2.runtimes = ['runc']
+        node2.enable_cpu_pinning = False
         node3 = objects.ComputeNode(self.context)
         node3.cpus = 48
         node3.cpu_used = 0.0
         node3.mem_total = 1024 * 128
         node3.mem_used = 1024 * 4
         node3.mem_free = 1024 * 124
+        node3.mem_available = 1024 * 124
         node3.disk_total = 80
         node3.disk_used = 20
         node3.hostname = 'host3'
-        node3.numa_topology = None
+        node3.numa_topology = numa
         node3.labels = {}
         node3.pci_device_pools = None
         node3.disk_quota_supported = True
+        node3.runtimes = ['runc']
+        node3.enable_cpu_pinning = False
         node4 = objects.ComputeNode(self.context)
         node4.cpus = 48
         node4.cpu_used = 0.0
         node4.mem_total = 1024 * 128
         node4.mem_used = 1024 * 4
         node4.mem_free = 1024 * 124
+        node4.mem_available = 1024 * 124
         node4.disk_total = 80
         node4.disk_used = 20
         node4.hostname = 'host4'
-        node4.numa_topology = None
+        node4.numa_topology = numa
         node4.labels = {}
         node4.pci_device_pools = None
         node4.disk_quota_supported = True
+        node4.runtimes = ['runc']
+        node4.enable_cpu_pinning = False
         nodes = [node1, node2, node3, node4]
         mock_compute_list.return_value = nodes
 
