@@ -15,7 +15,6 @@ import functools
 import shutil
 import six
 
-
 from oslo_log import log as logging
 from oslo_utils import excutils
 from oslo_utils import fileutils
@@ -24,6 +23,7 @@ from stevedore import driver as stevedore_driver
 from zun.common import exception
 from zun.common.i18n import _
 from zun.common import mount
+from zun.common import utils
 import zun.conf
 from zun.volume import cinder_api
 from zun.volume import cinder_workflow
@@ -33,12 +33,11 @@ LOG = logging.getLogger(__name__)
 CONF = zun.conf.CONF
 
 
-def driver(*args, **kwargs):
-    name = CONF.volume.driver
-    LOG.info("Loading volume driver '%s'", name)
+def driver(driver_name, *args, **kwargs):
+    LOG.info("Loading volume driver '%s'", driver_name)
     volume_driver = stevedore_driver.DriverManager(
         "zun.volume.driver",
-        name,
+        driver_name,
         invoke_on_load=True,
         invoke_args=args,
         invoke_kwds=kwargs).driver
@@ -80,9 +79,50 @@ class VolumeDriver(object):
     def bind_mount(self, *args, **kwargs):
         raise NotImplementedError()
 
-    def get_volume_status(self, *args, **kwargs):
+    def is_volume_available(self, context, volmap):
         raise NotImplementedError()
 
+    def is_volume_deleted(self, context, volmap):
+        raise NotImplementedError()
+
+
+class Local(VolumeDriver):
+
+    supported_providers = ['local']
+
+    @validate_volume_provider(supported_providers)
+    def attach(self, context, volmap):
+        mountpoint = mount.get_mountpoint(volmap.volume.uuid)
+        fileutils.ensure_tree(mountpoint)
+        filename = '/'.join([mountpoint, volmap.volume.uuid])
+        with open(filename, 'wb') as fd:
+            content = utils.decode_file_data(volmap.contents)
+            fd.write(content)
+
+    def _remove_local_file(self, volmap):
+        mountpoint = mount.get_mountpoint(volmap.volume.uuid)
+        shutil.rmtree(mountpoint)
+
+    @validate_volume_provider(supported_providers)
+    def detach(self, context, volmap):
+        self._remove_local_file(volmap)
+
+    @validate_volume_provider(supported_providers)
+    def delete(self, context, volmap):
+        self._remove_local_file(volmap)
+
+    @validate_volume_provider(supported_providers)
+    def bind_mount(self, context, volmap):
+        mountpoint = mount.get_mountpoint(volmap.volume.uuid)
+        filename = '/'.join([mountpoint, volmap.volume.uuid])
+        return filename, volmap.container_path
+
+    def is_volume_available(self, context, volmap):
+        return True, False
+
+    def is_volume_deleted(self, context, volmap):
+        return True, False
+
 
 class Cinder(VolumeDriver):
 
@@ -91,53 +131,87 @@ class Cinder(VolumeDriver):
     ]
 
     @validate_volume_provider(supported_providers)
-    def attach(self, context, volume):
+    def attach(self, context, volmap):
         cinder = cinder_workflow.CinderWorkflow(context)
-        devpath = cinder.attach_volume(volume)
+        devpath = cinder.attach_volume(volmap)
         try:
-            self._mount_device(volume, devpath)
+            self._mount_device(volmap, devpath)
         except Exception:
             with excutils.save_and_reraise_exception():
                 LOG.exception("Failed to mount device")
                 try:
-                    cinder.detach_volume(volume)
+                    cinder.detach_volume(volmap)
                 except Exception:
                     LOG.exception("Failed to detach volume")
 
-    def _mount_device(self, volume, devpath):
-        mountpoint = mount.get_mountpoint(volume.volume_id)
+    def _mount_device(self, volmap, devpath):
+        mountpoint = mount.get_mountpoint(volmap.volume.uuid)
         fileutils.ensure_tree(mountpoint)
         mount.do_mount(devpath, mountpoint, CONF.volume.fstype)
 
     @validate_volume_provider(supported_providers)
-    def detach(self, context, volume):
-        self._unmount_device(volume)
+    def detach(self, context, volmap):
+        self._unmount_device(volmap)
         cinder = cinder_workflow.CinderWorkflow(context)
-        cinder.detach_volume(volume)
+        cinder.detach_volume(context, volmap)
 
     @validate_volume_provider(supported_providers)
-    def delete(self, context, volume):
-        self._unmount_device(volume)
+    def delete(self, context, volmap):
         cinder = cinder_workflow.CinderWorkflow(context)
-        cinder.delete_volume(volume)
+        cinder.delete_volume(volmap)
 
-    def _unmount_device(self, volume):
-        if hasattr(volume, 'connection_info'):
-            mountpoint = mount.get_mountpoint(volume.volume_id)
+    def _unmount_device(self, volmap):
+        if hasattr(volmap, 'connection_info'):
+            mountpoint = mount.get_mountpoint(volmap.volume.uuid)
             mount.do_unmount(mountpoint)
             shutil.rmtree(mountpoint)
 
     @validate_volume_provider(supported_providers)
-    def bind_mount(self, context, volume):
-        mountpoint = mount.get_mountpoint(volume.volume_id)
-        return mountpoint, volume.container_path
+    def bind_mount(self, context, volmap):
+        mountpoint = mount.get_mountpoint(volmap.volume.uuid)
+        return mountpoint, volmap.container_path
 
     @validate_volume_provider(supported_providers)
-    def get_volume_status(self, context, volume):
+    def get_volume_status(self, context, volmap):
         ca = cinder_api.CinderAPI(context)
-        return ca.get(volume.volume_id).status
+        return ca.get(volmap.cinder_volume_id).status
 
     @validate_volume_provider(supported_providers)
-    def check_multiattach(self, context, volume):
+    def check_multiattach(self, context, volmap):
         ca = cinder_api.CinderAPI(context)
-        return ca.get(volume.volume_id).multiattach
+        return ca.get(volmap.cinder_volume_id).multiattach
+
+    @validate_volume_provider(supported_providers)
+    def is_volume_available(self, context, volmap):
+        status = self.get_volume_status(context, volmap)
+        if status == 'available':
+            is_available = True
+            is_error = False
+        elif status == 'in-use':
+            multiattach = self.check_multiattach(context, volmap)
+            is_available = multiattach
+            is_error = False
+        elif status == 'error':
+            is_available = False
+            is_error = True
+        else:
+            is_available = False
+            is_error = False
+
+        return is_available, is_error
+
+    @validate_volume_provider(supported_providers)
+    def is_volume_deleted(self, context, volmap):
+        try:
+            volume = cinder_api.CinderAPI(context).search_volume(
+                volmap.cinder_volume_id)
+            is_deleted = False
+            # Cinder volume error states: 'error', 'error_deleting',
+            # 'error_backing-up', 'error_restoring', 'error_extending',
+            # all of which start with 'error'
+            is_error = True if 'error' in volume.status else False
+        except exception.VolumeNotFound:
+            is_deleted = True
+            is_error = False
+
+        return is_deleted, is_error
